@@ -1,4 +1,4 @@
-;;; these are CLM test instruments
+;;; these are (ancient!) CLM test instruments
 
 (provide 'snd-clm23.scm)
 (if (provided? 'snd)
@@ -40,9 +40,6 @@
 			  (sin (phases i))))))
     sum))
 
-
-(define open-input make-file->sample)
-
 (define two-pi (* 2 pi))
 
 (define (simple-out beg dur freq amp)
@@ -56,8 +53,8 @@
   (let ((fm-index (hz->radians (* index mc-ratio freq))))
     (let ((start (seconds->samples beg))
 	  (end (seconds->samples (+ beg dur)))
-	  (cr (make-oscil freq))                     ; our carrier
-	  (md (make-oscil (* freq mc-ratio)))        ; our modulator
+	  (cr (make-oscil freq))                     ; carrier
+	  (md (make-oscil (* freq mc-ratio)))        ; modulator
 	  (ampf (make-env (or amp-env '(0 0 .5 1 1 0)) :scaler amp :duration dur))
 	  (indf (make-env (or index-env '(0 0 .5 1 1 0)) :scaler fm-index :duration dur)))
       (do ((i start (+ i 1)))
@@ -78,11 +75,10 @@
 	(if (> revb 0.0) (outb i (* revb val) *reverb*))))))
 
 (define (simple-ssb beg dur freq amp)
-  (let ((os (make-ssb-am freq))
-	(start (seconds->samples beg))
+  (let ((start (seconds->samples beg))
 	(end (seconds->samples (+ beg dur)))
 	(arr (make-vector 3)))
-    (set! (arr 0) os)
+    (set! (arr 0) (make-ssb-am freq))
     (set! (arr 1) #f)
     (set! (arr 2) (make-ssb-am 660 40))
     (do ((k 0 (+ k 1)))
@@ -138,14 +134,12 @@
 (define (simple-osc beg dur freq amp)
   (let ((start (seconds->samples beg))
 	(end (seconds->samples (+ beg dur)))
-	(freqs (make-float-vector 20)))
-    (let ((obank (make-oscil-bank freqs 
-				  (let ((fv (make-float-vector 20 0.0)))
-				    (do ((i 0 (+ i 1)))
-					((= i 20) fv)
-				      (set! (freqs i) (hz->radians (* (+ i 1) 100)))))
-				  (make-float-vector 20 1.0)
-				  #t)))
+	(freqs (make-float-vector 20))
+	(fv (make-float-vector 20)))
+    (do ((i 0 (+ i 1)))
+	((= i 20))
+      (set! (freqs i) (hz->radians (* (+ i 1) freq))))
+    (let ((obank (make-oscil-bank freqs fv (make-float-vector 20 1.0) #t)))
       (set! amp (* 0.05 amp))
       (do ((i start (+ i 1))) 
 	  ((= i end))
@@ -398,12 +392,12 @@
   (let ((foflen (if (= *clm-srate* 22050) 100 200)))
     (let ((start (seconds->samples beg))
 	  (end (seconds->samples (+ beg dur)))
-	  (ampf (make-env :envelope (or ae (list 0 0 25 1 75 1 100 0)) :scaler amp :duration dur))
+	  (ampf (make-env :envelope (or ae '(0 0 25 1 75 1 100 0)) :scaler amp :duration dur))
 	  (frq0 (hz->radians f0))
 	  (frq1 (hz->radians f1))
 	  (frq2 (hz->radians f2))
 	  (vibr (make-oscil 6))
-	  (vibenv (make-env :envelope (or ve (list 0 1 100 1)) :scaler vib :duration dur))
+	  (vibenv (make-env :envelope (or ve '(0 1 100 1)) :scaler vib :duration dur))
 	  (win-freq (/ two-pi foflen))
 	  (wt0 (make-wave-train :size foflen :frequency frq)))
       (let ((foftab (mus-data wt0)))
@@ -417,7 +411,7 @@
 	(outa i (* (env ampf) (wave-train wt0 (* (env vibenv) (oscil vibr)))))))))
 
 (define (simple-amb beg dur freq amp)
-  (let ((os (if (> freq 1) (make-oscil freq) (make-rand freq)))
+  (let ((os ((if (> freq 1) make-oscil make-rand) freq))
 	(start (seconds->samples beg))
 	(end (seconds->samples (+ beg dur))))
     (do ((i start (+ i 1))) ((= i end))
@@ -535,17 +529,17 @@
 (define (simple-ina beg dur amp file)
   (let ((start (seconds->samples beg))
 	(end (seconds->samples (+ beg dur)))
-	(fil (open-input file)))
+	(fil (make-file->sample file)))
     (do ((i start (+ i 1))
 	 (ctr 0 (+ ctr 1)))
 	((= i end))
       (outa i (* amp (in-any ctr 0 fil))))))
 
 (define (simple-in-rev beg dur ampa ampb)
-  (let ((start (seconds->samples beg))
-	(end (seconds->samples (+ beg dur)))
-	(chns (mus-channels *reverb*)))
-    (if (or (> ampa 0.0) (> ampb 0.0))
+  (if (or (> ampa 0.0) (> ampb 0.0))
+      (let ((start (seconds->samples beg))
+	    (end (seconds->samples (+ beg dur)))
+	    (chns (mus-channels *reverb*)))
 	(if (or (zero? ampb) (= chns 1))
 	    (do ((i start (+ i 1))) ((= i end))
 	      (outa i (* ampa (ina i *reverb*))))
@@ -709,10 +703,10 @@
   (let ((start (seconds->samples beg))
 	(end (seconds->samples (+ beg dur)))
 	(os (make-oscil freq)))
-    (let ((sr2 (make-src :srate speed :input (lambda (dir) (oscil os)))))
-      (let ((sr1 (make-src :srate speed :input (lambda (dir) (src sr2)))))
-	(do ((i start (+ i 1))) ((= i end))
-	  (outa i (* amp (src sr1))))))))
+    (let ((sr1 (let ((sr2 (make-src :srate speed :input (lambda (dir) (oscil os)))))
+		 (make-src :srate speed :input (lambda (dir) (src sr2))))))
+      (do ((i start (+ i 1))) ((= i end))
+	(outa i (* amp (src sr1)))))))
 
 (define (sample-srl3 beg dur amp speed freq)
   (let ((start (seconds->samples beg))
@@ -865,10 +859,10 @@
 	    (format () "~A ~A~%" (xs 1) (mus-xcoeff flt 1))))
       (let ((data (mus-data flt)))
 	(if (> (data 0) 1.0) (format () "data overflow? ~A~%" (data 0))))
-      (let ((is intdat)
-	    (fs fltdat))
-	(if (not (= (is 1) 3)) (format () "intdat let: ~A~%" (is 1)))
-	(if (> (abs (- (fs 1) 3.14)) .001) (format () "fltdat let: ~A~%" (fs 1))))
+      (if (not (= (intdat 1) 3))
+          (format () "intdat let: ~A~%" (intdat 1)))
+      (if (> (abs (- (fltdat 1) 3.14)) 0.001)
+          (format () "fltdat let: ~A~%" (fltdat 1)))
       (outa i (* amp (filter flt (oscil os)))))))
 
 (define (sample-arrintp beg dur freq amp)
@@ -887,79 +881,21 @@
 	  ((= i end))
 	(outa i (* amp (array-interp arr loc) (oscil os)))))))
 
-(define (sample-if beg dur freq amp)
-  (let ((os (make-oscil freq))
-	(start (seconds->samples beg))
-	(end (seconds->samples (+ beg dur)))
-	(k -123)
-	(j 0)
-	(bool #t))
-    (do ((i start (+ i 1))) ((= i end))
-      (if (and (= i start) (not (= k -123))) (format () "init k: ~A~%" k))
-      (if (not bool) (format () "bool: ~A~%" bool))
-      (set! j (if bool 1 2))
-      (if (not (= j 1)) (format () "if expr: ~A~%" j))
-      (set! j (if bool 3 4))
-      (if (not (= j 3)) (format () "if statement: ~A~%" j))
-      (if (integer? k) (set! j 5))
-      (if (not (= j 5)) (format () "int k? ~A ~A~%" (integer? k) j))
-      (if (= j k) (set! j 6))
-      (if (= j 6) (format () "j if false: ~A~%" j))
-      (set! j (+ k (if (= j k) 7 8)))
-      (if (not (= j (+ k 8))) (format () "if false expr: ~A ~A~%" j k))
-      (set! j (if (> j -1234) (if (> k -1234) 9 10) 11))
-      (if (not (= j 9)) (format () "if 2 expr: ~A~%" j))
-      (set! j (if (> j -1234) (begin (set! k 0) 12) 13))
-      (if (not (= j 12)) (format () "if begin expr: ~A~%" j))
-      (set! j 123)
-      (case j
-	((0) (set! k -1))
-	((12) (set! k -2))
-	((123) (set! k -3))
-	(else (set! k -4)))
-      (if (not (= k -3)) (format () "cond: ~A ~A~%" j k))
-      (set! k (case j
-		((0) -4)
-		((12) -5)
-		(else -6)))
-      (if (not (= k -6)) (format () "cond expr: ~A ~A~%" j k))
-      (set! k (let ((a 123))
-		(if (> a 0)
-		    20
-		    32)))
-      (if (not (= k 20)) (format () "let expr: ~A ~A~%" j k))
-      (let ((a 123))
-	(set! k a))
-      (if (not (= k 123)) (format () "let: ~A ~A~%" j k))
-      (set! k 123)
-      (set! bool (= k 123))
-      (if (not bool) (format () "bool expr: ~A~%" bool))
-      (set! bool (if (= k 123) (> k 0) (< k 0)))
-      (if (not bool) (format () "if bool expr: ~A~%" bool))
-      (set! j 0)
-      (set! k (do ((m 0 (+ 1 m)))
-		  ((= m 3) j)
-		(set! j (+ j 1))))
-      (if (not (= k 3)) (format () "do expr: ~A~%" k))
-					;	 (dotimes (m 2)
-					;		  (set! k (- k 1)))
-					;	 (if (not (= k 1)) (format () "dotimes: ~A~%" k))
-      (outa i (* amp (oscil os))))))
-
 (define (sample-arrfile beg dur freq amp)
   (let ((os (make-oscil freq))
 	(start (seconds->samples beg))
 	(end (seconds->samples (+ beg dur)))
-	(arr (make-float-vector 100))
-	(ctr 0)
-	(dir 1))
+	(arr (make-float-vector 100)))
     (do ((i 0 (+ i 1)))
 	((= i 100))
-      (set! (arr i) (* amp (+ -.5 (* i .01)))))
+      (set! (arr i) (* amp (- (* i .01) 0.5))))
     (array->file "testx.data" arr 100 22050 1)
     (fill! arr 0.0)
     (file->array "testx.data" 0 0 100 arr)
-    (do ((i start (+ i 1))) ((= i end))
+    (do ((ctr 0)
+	 (dir 1)
+	 (i start (+ i 1))) 
+	((= i end))
       (outa i (* (arr ctr) (oscil os)))
       (set! ctr (+ ctr dir))
       (if (>= ctr 99) (set! dir -1)
@@ -1089,7 +1025,6 @@
   (sample-ardcl 5.5 .2 440 .1)
   (sample-flt 6 .2 440 .1)
   (sample-arrintp 6.25 .2 440 .1)
-  (sample-if 6.5 .2 440 .1)
   (sample-arrfile 6.75 .2 440 .15)
   (sample-pvoc5 7 .2 .1 256 "oboe.snd" 440.0)
   )
@@ -1136,24 +1071,23 @@
 						     ((= k size))
 						   (float-vector-set! paincrs buf (* (in-data k) (window k)))
 						   (set! buf (+ buf 1))
-						   (if (>= buf size) (set! buf 0)))
-						 (set! filptr (+ filptr D))
-						 (mus-fft paincrs freqs size 1)
-						 (rectangular->polar paincrs freqs)
-						 #f))
+						   (if (>= buf size) (set! buf 0))))
+					       (set! filptr (+ filptr D))
+					       (mus-fft paincrs freqs size 1)
+					       (rectangular->polar paincrs freqs)
+					       #f)
 				    :edit (lambda (pv)
-					    (let ((pscl (/ 1.0 D))
-						  (kscl (/ two-pi size)))
-					      (do ((k 0 (+ k 1))
-						   (ks 0.0 (+ ks kscl)))
-						  ((= k N2))
-						(let* ((freq (freqs k))
-						       (diff (- freq (lastphases k))))
-						  (set! (lastphases k) freq)
-						  (if (> diff pi) (set! diff (- diff two-pi)))
-						  (if (< diff (- pi)) (set! diff (+ diff two-pi)))
-						  (float-vector-set! freqs k (+ (* diff  pscl) ks))))
-					      #f))
+					    (do ((pscl (/ 1.0 D))
+						 (kscl (/ two-pi size))
+						 (k 0 (+ k 1))
+						 (ks 0.0 (+ ks kscl)))
+						((= k N2) #f)
+					      (let* ((freq (freqs k))
+						     (diff (- freq (lastphases k))))
+						(set! (lastphases k) freq)
+						(if (> diff pi) (set! diff (- diff two-pi)))
+						(if (< diff (- pi)) (set! diff (+ diff two-pi)))
+						(float-vector-set! freqs k (+ (* diff  pscl) ks)))))
 				    :synthesize (lambda (pv)
 						  (float-vector-add! amps paincrs)
 						  (float-vector-add! ppincrs freqs)
@@ -1170,119 +1104,6 @@
 	  ((= i end))
 	(outa i (* amp (phase-vocoder sr))))))))
 
-(define (or1)
-  (let ((e1 (make-env '(0 0 1 1) :length 10))
-	(e2 (make-env '(0 1 1 0) :length 10))
-	(e3 #f)
-	(ok1 0.0))
-    (do ((i 0 (+ i 1)))
-	((= i 1))
-      (set! ok1 0.0)
-      (if (or e1 e2)
-	  (set! ok1 (+ ok1 (env e1)))
-	  (format () ";or1 a~%"))
-      (if (not (or e1 e2))
-	  (format () ";or1 1~%"))
-      (if (and e1 e2)
-	  (set! ok1 (+ ok1 (env e2)))
-	  (format () ";or1 b~%"))
-      (if (not (and e1 e2))
-	  (format () ";or1 2~%"))
-      (if (or e3 e1 e2)
-	  (mus-reset e2) ; resets e2 -> 1.0
-	  (format () ";or1 c~%"))
-      (if (and e1 e3 e2)
-	  (format () ";or1 3~%"))
-      (if (not e1)
-	  (format () ";or1 4~%"))
-      (if (< (abs ok1) .001)
-	  (format () ";or1 ok1: ~A~%" ok1)))))
-
-(define (or2)
-  (let ((e1 (make-env '(0 0 1 1) :length 10))
-	(e2 (make-env '(0 1 1 0) :length 10))
-	(e3 #f)
-	(ok1 0.0)
-	(oki 0)
-	(okb #f))
-    (do ((i 0 (+ i 1)))
-	((= i 1))
-      (set! ok1 0.0)
-      (set! oki (+ oki 1))
-      (set! okb #t)
-      (if (or e1 e2)
-	  (set! ok1 (+ ok1 (env e1)))
-	  (format () ";or2 a~%"))
-      (if (not (or e1 e2 okb))
-	  (format () ";or2 1~%"))
-      (if (and e1 e2)
-	  (set! ok1 (+ ok1 (env e2)))
-	  (format () ";or2 b~%"))
-      (if (not (and e1 e2))
-	  (format () ";or2 2~%"))
-      (if (or e3 e1 e2)
-	  (mus-reset e2) ; resets e2 -> 1.0
-	  (format () ";or2 c~%"))
-      (if (and e1 e3 e2)
-	  (format () ";or2 3~%"))
-      (if (not e1)
-	  (format () ";or2 4~%"))
-      (if (< (abs ok1) .001)
-	  (format () ";or1 ok1: ~A~%" ok1)))))
-
-(define (or3)
-  (let ((e1 (make-env '(0 0 1 1) :length 10))
-	(i1 (make-vector 3 32))
-	(f1 (make-float-vector 3 3.14))
-	(i2 (make-vector 3 3))
-	(f2 (make-vector 3 1.5))
-	(ok1 0.0)
-	(oki 0))
-    (do ((i 0 (+ i 1)))
-	((= i 1))
-      (cond (e1 (set! ok1 (+ ok1 (env e1))))
-	    (#t (format () ";or3 1~%")))
-      (if (or f1 f2)
-	  (set! ok1 (+ ok1 (f2 0)))
-	  (format () ";or3 a~%"))
-      (if (not (or f2 f1))
-	  (format () ";or3 2~%"))
-      (if (and f2 f1)
-	  (set! ok1 (+ ok1 (f1 1)))
-	  (format () ";or3 b~%"))
-      (if (or i1 i2)
-	  (set! oki (+ oki (i2 0)))
-	  (format () ";or3 d~%"))
-      (if (not (or i2 i1))
-	  (format () ";or3 3~%"))
-      (if (and i2 i1)
-	  (set! oki (+ oki (i1 1)))
-	  (format () ";or3 e~%")))))
-
-(define (or4)
-  (let ((e1 (make-env '(0 0 1 1) :length 10))
-	(e2 (make-env '(0 1 1 0) :length 10))
-  	(i1 (make-vector 3 32))
-	(f1 (make-float-vector 3 3.14))
-	(i2 (make-vector 3 3))
-	(f2 (make-vector 3 1.5))
-	(oki 0))
-    (do ((i 0 (+ i 1)))
-	((= i 1))
-      (if (or (and e1 e2)
-	      (and f1 f2)
-	      (and i1 i2))
-	  (set! oki (+ oki 1))
-	  (format () ";or4 a~%"))
-      (if (and (or f1 f2)
-	       (not (or i1 i2))
-	       (or e1 e2))
-	  (format () ";or4 1~%"))
-      (cond ((not f1) (format () ";or4 4~%"))
-	    ((not e1) (format () ";or4 3~%"))
-	    ((not e2) (format () ";or4 2~%"))
-	    (else (set! oki (+ oki 1)))))))
-
 
 ;;; --------------------------------------------------------------------------------
 ;;;
@@ -1300,8 +1121,7 @@
         (modulator-phase-incr (hz->radians (* mc-ratio freq))))
     (do ((i beg (+ i 1)))
 	((= i end))
-      (let* ((modulation (* index (sin modulator-phase)))
-	     (pm-val (* amp (sin (+ carrier-phase modulation))))) 
+      (let ((pm-val (* amp (sin (+ carrier-phase (* index (sin modulator-phase)))))))
 	;; no integration in phase modulation (it's a phase change)
 	(set! carrier-phase (+ carrier-phase carrier-phase-incr))
 	(set! modulator-phase (+ modulator-phase modulator-phase-incr))
@@ -1396,7 +1216,7 @@
 	      ((= i n))
 	    (set! (modulators i) (hz->radians (* freq (mc-ratios i) (mod-phases i))))
 	    (set! (fm-indices i) (hz->radians (* freq (indexes i) (mc-ratios i)))))
-	  (let ((ob (make-oscil-bank modulators (make-float-vector n 0.0) fm-indices #t)))
+	  (let ((ob (make-oscil-bank modulators (make-float-vector n) fm-indices #t)))
 	    (do ((i start (+ i 1)))
 		((= i end))
 	      (outa i (* amp (oscil cr (oscil-bank ob))))))))))
@@ -1408,7 +1228,7 @@
 	(frq-scl (hz->radians frequency)))
     (let ((maxdev (* frq-scl fm-index)))
       (let ((index1 (* maxdev (/ 5.0 (log frequency))))
-	    (index2 (* maxdev 3.0 (/ (- 8.5 (log frequency)) (+ 3.0 (/ frequency 1000)))))
+	    (index2 (/ (* maxdev 3.0 (- 8.5 (log frequency))) (+ 3.0 (/ frequency 1000))))
 	    (index3 (* maxdev (/ 4.0 (sqrt frequency))))
 	    (carrier (make-oscil frequency))
 	    (fmosc1 (make-oscil frequency))
@@ -1437,7 +1257,7 @@
 	(md (make-oscil (* freq modrat)))
 	(ca (make-oscil (* freq casrat) caspha))
 	(fm-ind0 (hz->radians (* modind modrat freq)))
-	(fm-ind1 (hz->radians (* casind (/ casrat modrat) freq))))
+	(fm-ind1 (hz->radians (/ (* casind casrat freq) modrat))))
     (do ((i start (+ i 1)))
 	((= i end))
       (outa i (* amp 
@@ -1597,18 +1417,6 @@
 	((= i end))
       (outa i (* amp (oscil os (env frqe)))))))
 
-(definstrument (sndclmdoc-simple-fm beg dur freq amp mc-ratio index amp-env index-env)
-  (let ((fm-index (hz->radians (* index mc-ratio freq))))
-    (let ((start (seconds->samples beg))
-	  (end (seconds->samples (+ beg dur)))
-	  (cr (make-oscil freq))                     ; carrier
-	  (md (make-oscil (* freq mc-ratio)))        ; modulator
-	  (ampf (make-env (or amp-env '(0 0  .5 1  1 0)) :scaler amp :duration dur))
-	  (indf (make-env (or index-env '(0 0  .5 1  1 0)) :scaler fm-index :duration dur)))
-      (do ((i start (+ i 1)))
-	  ((= i end))
-	(outa i (* (env ampf) (oscil cr (* (env indf) (oscil md)))))))))
-
 (define (sndclmdoc-simple-add beg dur freq amp)
   (let ((start (seconds->samples beg))
 	(end (seconds->samples (+ beg dur)))
@@ -1648,12 +1456,11 @@
     (if (or (null? loop-data)
 	    (<= (cadr loop-data) (car loop-data)))
 	(error 'no-loop-positions)
-	(let* ((loop-start (car loop-data))
-	       (loop-end (cadr loop-data))
-	       (loop-length (+ 1 (- loop-end loop-start)))
-	       (sound-section (float-vector-scale! (file->array sound 0 loop-start loop-length (make-float-vector loop-length)) amp))
-	       (original-loop-duration (/ loop-length (srate sound)))
-	       (tbl (make-table-lookup :frequency (/ freq original-loop-duration) :wave sound-section)))
+	(let ((tbl (let* ((loop-start (car loop-data))
+			  (loop-length (- (+ (cadr loop-data) 1) loop-start))
+			  (sound-section (float-vector-scale! (file->array sound 0 loop-start loop-length (make-float-vector loop-length)) amp)))
+		     (make-table-lookup :frequency (/ (* freq (srate sound)) loop-length)
+					:wave sound-section))))
 	  ;; "freq" here is how fast we read (transpose) the sound -- 1.0 returns the original
 	  (do ((i beg (+ i 1)))
 	      ((= i end))
@@ -1707,10 +1514,13 @@
 		      (polynomial cos-coeffs ax))))))))
 
 (definstrument (sndclmdoc-bl-saw start dur frequency order)
-  (let ((norm (cond ((= order 1) 1.0)      ; these peak amps were determined empirically
-		    ((= order 2) 1.3)      ;   actual limit is supposed to be pi/2 (G&R 1.441)
-		    ((< order 9) 1.7)      ;   but Gibbs phenomenon pushes it to 1.851
-		    (else 1.9)))           ;   if order>25, numerical troubles -- use table-lookup
+  (let ((norm (cond ((assoc order '((1 . 1.0) (2 . 1.3)) =) => cdr)
+		    ((< order 9) 1.7)
+		    (else 1.9)))
+	;; these peak amps were determined empirically
+	;;   actual limit is supposed to be pi/2 (G&R 1.441)
+	;;   but Gibbs phenomenon pushes it to 1.851
+	;;   if order>25, numerical troubles -- use table-lookup
 	(freqs ()))
     (do ((i 1 (+ i 1)))
 	((> i order))
@@ -1757,15 +1567,14 @@
   (float-vector 0.0 (hz->radians frequency) (* 1.0 n)))
 
 (define (sndclmdoc-sum-of-odd-sines gen fm)
-  (let* ((angle (gen 0))
-	 (a2 (* angle 0.5))
-	 (n (gen 2))
-	 (den (* n (sin a2)))
-	 (result (if (< (abs den) 1.0e-9)
-		     0.0
-		     (/ (* (sin (* n a2)) 
-			   (sin (* (+ 1 n) a2)))
-			den))))
+  (let ((result (let* ((a2 (* (gen 0) 0.5))
+		       (n (gen 2))
+		       (den (* n (sin a2))))
+		  (if (< (abs den) 1.0e-9)
+		      0.0
+		      (/ (* (sin (* n a2)) 
+			    (sin (* (+ 1 n) a2)))
+			 den)))))
     (set! (gen 0) (+ (gen 0) (gen 1) fm))
     result))
 
@@ -1822,12 +1631,12 @@
     (let ((foftab (make-float-vector foflen)))
       (let ((start (seconds->samples beg))
 	    (end (seconds->samples (+ beg dur)))
-	    (ampf (make-env :envelope (or ae (list 0 0 25 1 75 1 100 0)) :scaler amp :duration dur))
+	    (ampf (make-env :envelope (or ae '(0 0 25 1 75 1 100 0)) :scaler amp :duration dur))
 	    (frq0 (hz->radians f0))
 	    (frq1 (hz->radians f1))
 	    (frq2 (hz->radians f2))
 	    (vibr (make-oscil 6))
-	    (vibenv (make-env :envelope (or ve (list 0 1 100 1)) :scaler vib :duration dur))
+	    (vibenv (make-env :envelope (or ve '(0 1 100 1)) :scaler vib :duration dur))
 	    (win-freq (/ (* 2 pi) foflen))
 	    (wt0 (make-wave-train :wave foftab :frequency frq)))
 	(do ((i 0 (+ i 1)))
@@ -1915,11 +1724,9 @@
       (outa i (convolve ff)))))
 
 (definstrument (sndclmdoc-granulate-sound file beg dur (orig-beg 0.0) (exp-amt 1.0))
-  (let* ((f-srate (srate file))
-	 (f-start (round (* f-srate orig-beg)))
-         (f (make-readin file :start f-start))
-	 (st (seconds->samples beg))
-	 (new-dur (or dur (- (mus-sound-duration file) orig-beg))))
+  (let ((f (make-readin file :start (round (* (srate file) orig-beg))))
+	(st (seconds->samples beg))
+	(new-dur (or dur (- (mus-sound-duration file) orig-beg))))
     (let ((exA (make-granulate :input f :expansion exp-amt))
 	  (nd (+ st (seconds->samples new-dur))))
       (do ((i st (+ i 1)))
@@ -2200,9 +2007,9 @@
     (lambda (gen input)
       (let-set! gen 'input input)
       (with-let gen
-	(let* ((modphase (* ratio phase))
-	       (result (* (exp (* r2 (cos modphase)))
-			  (sin (+ phase (* r1 (sin modphase)))))))
+	(let ((result (let ((modphase (* ratio phase)))
+			(* (exp (* r2 (cos modphase)))
+			   (sin (+ phase (* r1 (sin modphase))))))))
 	  (set! phase (+ phase input freq))
 	  result)))))
 
@@ -2211,9 +2018,9 @@
     (lambda (gen input)
       (let-set! gen 'input input)
       (with-let gen
-	(let* ((modphase (* ratio phase))
-	       (result (* (exp (- (* r1 (cos modphase)) r3))
-			  (sin (+ phase (* r2 (sin modphase)))))))
+	(let ((result (let ((modphase (* ratio phase)))
+			(* (exp (- (* r1 (cos modphase)) r3))
+			   (sin (+ phase (* r2 (sin modphase))))))))
 	  (set! phase (+ phase input freq))
 	  result)))))
 
@@ -2248,15 +2055,6 @@
       (if (< i end)
 	  (loop (+ i 1))))))
 
-(define (simp-tail-recursion beg dur freq amp)
-  (let ((o (make-oscil freq))
-	 (start (seconds->samples beg))
-	 (end (seconds->samples (+ beg dur))))
-    (define (simper i)
-      (outa i (* amp (oscil o)))
-      (if (< i end)
-	  (simper (+ i 1))))
-    (simper start)))
 
 
 ;;; --------------------------------------------------------------------------------
@@ -2304,7 +2102,7 @@
 	(outa i (my-oscil osc 0.0)))))
   (with-sound () 
     (sndclmdoc-simp-5 0 10000 440 .1 '(0 0 1 1)) ; sweep up an octave
-    (sndclmdoc-simple-fm 1 1 440 .1 2 1.0)
+    (simple-fm 1 1 440 .1 2 1.0)
     (sndclmdoc-simple-add 2 1 220 .3)
     (sndclmdoc-mapenv 3 1 440 .4 '(0 0 50 1 75 0 86 .5 100 0)))
   (if (file-exists? "/home/bil/sf1/forest.aiff")
@@ -2319,11 +2117,11 @@
     (sndclmdoc-tritri 0 1 1000.0 0.5 0.1 0.01) ; sci-fi laser gun
     (sndclmdoc-tritri 1 1 4000.0 0.7 0.1 0.01)) ; a sparrow?
   (with-sound (:srate 22050) (sndclmdoc-shift-pitch 0 3 "oboe.snd" 1108.0)) ; 1.7
-  (let* ((sound "oboe.snd") ; 1.8
-	 (mx (maxamp sound))
-	 (dur (mus-sound-duration sound)))
-    (with-sound (:scaled-to mx :srate 22050) 
-      (sndclmdoc-repitch 0 dur sound 554 1000)))
+  (let ((sound "oboe.snd")) ; 1.8
+    (let ((mx (maxamp sound))
+	  (dur (mus-sound-duration sound)))
+      (with-sound (:scaled-to mx :srate 22050) 
+	(sndclmdoc-repitch 0 dur sound 554 1000))))
   (with-sound () (sndclmdoc-fofins 0 1 270 .2 .001 730 .6 1090 .3 2440 .1)) ; "Ahh"
   (with-sound () ; one of JC's favorite demos
     (sndclmdoc-fofins 0 4 270 .2 0.005 730 .6 1090 .3 2440 .1 '(0 0 40 0 75 .2 100 1) 
@@ -2360,12 +2158,9 @@
     (sndclmdoc-space "pistol.snd" 0 1 :distance-env '(0 1 1 2) :degree-env '(0 0 1 90)))
   
   (with-sound ()
-    (let ((gen (sndclmdoc-make-sum-of-odd-sines 440.0 10)))
-      (sndclmdoc-sum-of-odd-sines gen 0.0))
-    (let ((gen (sndclmdoc-make-sinc-train 440.0)))
-      (sndclmdoc-sinc-train gen))
-    (let ((gen (sndclmdoc-make-moving-max)))
-      (sndclmdoc-moving-max gen 0.1))
+    (sndclmdoc-sum-of-odd-sines (sndclmdoc-make-sum-of-odd-sines 440.0 10) 0.0)
+    (sndclmdoc-sinc-train (sndclmdoc-make-sinc-train 440.0))
+    (sndclmdoc-moving-max (sndclmdoc-make-moving-max) 0.1)
     (sndclmdoc-asy 0 .1 440 .1 1.0)
     (sndclmdoc-simple-table 1000)
     (sndclmdoc-simple-f2s .1 .1 .1 "oboe.snd")
@@ -2387,10 +2182,8 @@
   (with-sound (:channels 4) (sndclmdoc-simple-dloc 0 2 440 .5))
   (with-sound () (when? 0 4 2.0 8.0 "1a.snd"))
   (with-sound () (move-formants 0 "oboe.snd" 2.0 0.99 '(0 1200 1.6 2400 2 1400) 4))
-  (test-filter (make-one-zero 0.5 0.5))
-  (test-filter (make-one-pole 0.1 -0.9))
-  (test-filter (make-two-pole 0.1 0.1 0.9))
-  (test-filter (make-two-zero 0.5 0.2 0.3))
+
+  (for-each test-filter (vector (make-one-zero 0.5 0.5) (make-one-pole 0.1 -0.9) (make-two-pole 0.1 0.1 0.9) (make-two-zero 0.5 0.2 0.3)))
   
   (with-sound (:scaled-to .5) ; .875
     (flux 0 "oboe.snd" 10.0 '(1.0 1.25 1.5) '(1.0 1.333 1.6))
@@ -2404,8 +2197,7 @@
     (sndscm-osc-fm 0 1 440 .1 1 1)
     (sndscm-osc1-fm 0 1 440 .1 1)
     (sndscm-osc2-fm 0 1 440.0 .1 1)
-    (simp-named-let 0 .01 440 .1)
-    (simp-tail-recursion 0 .01 440 .1))
+    (simp-named-let 0 .01 440 .1))
   
   (with-sound () 
     (let ((gen (make-dsp-asyfm :freq 2000 :ratio .1))) 
