@@ -28,7 +28,7 @@
 end: (window-envelope 1.0 3.0 '(0.0 0.0 5.0 1.0)) -> '(1.0 0.2 3.0 0.6)"))
     (lambda (beg end e)
       (let ((nenv ())
-	    (lasty (if e (cadr e) 0.0))
+	    (lasty (if (pair? e) (cadr e) 0.0))
 	    (len (length e)))
 	(call-with-exit
 	 (lambda (return-early)               
@@ -37,21 +37,19 @@ end: (window-envelope 1.0 3.0 '(0.0 0.0 5.0 1.0)) -> '(1.0 0.2 3.0 0.6)"))
 	     (let ((x (e i))
 		   (y (e (+ i 1))))
 	       (set! lasty y)
-	       (if (null? nenv)
-		   (if (>= x beg)
-		       (begin
-			 (set! nenv (append nenv (list beg (envelope-interp beg e))))
-			 (if (not (= x beg))
-			     (if (>= x end)
-				 (return-early (append nenv (list end (envelope-interp end e))))
-				 (set! nenv (append nenv (list x y)))))))
-		   (if (<= x end)
-		       (begin
-			 (set! nenv (append nenv (list x y)))
-			 (if (= x end)
-			     (return-early nenv)))
-		       (if (> x end)
-			   (return-early (append nenv (list end (envelope-interp end e)))))))))
+	       (cond ((null? nenv)
+		      (when (>= x beg)
+			(set! nenv (append nenv (list beg (envelope-interp beg e))))
+			(if (not (= x beg))
+			    (if (>= x end)
+				(return-early (append nenv (list end (envelope-interp end e))))
+				(set! nenv (append nenv (list x y)))))))
+		     ((<= x end)
+		      (set! nenv (append nenv (list x y)))
+		      (if (= x end) (return-early nenv)))
+		     ((> x end)
+		      (return-early
+		       (append nenv (list end (envelope-interp end e))))))))
 	   (append nenv (list end lasty))))))))
 
 
@@ -61,26 +59,24 @@ end: (window-envelope 1.0 3.0 '(0.0 0.0 5.0 1.0)) -> '(1.0 0.2 3.0 0.6)"))
   (let ((documentation "(map-envelopes func env1 env2) maps func over the breakpoints in env1 and env2 returning a new envelope"))
     (lambda (op e1 e2)
       (let ((xs ()))
-	(letrec ((at0 
-		  (lambda (e)
-		    (let* ((diff (car e))
-			   (len (length e))
-			   (lastx (e (- len 2)))
-			   (newe (copy e)))
-		      (do ((i 0 (+ i 2)))
-			  ((>= i len) newe)
-			(let ((x (/ (- (newe i) diff) lastx)))
-			  (set! xs (cons x xs))
-			  (set! (newe i) x))))))
-		 (remove-duplicates
-		  (lambda (lst)
-		    (letrec ((rem-dup
-			      (lambda (lst nlst)
-				(cond ((null? lst) nlst)
-				      ((member (car lst) nlst) (rem-dup (cdr lst) nlst))
-				      (else (rem-dup (cdr lst) (cons (car lst) nlst)))))))
-		      (rem-dup lst ())))))
-	  
+	(let ((at0 
+	       (lambda (e)
+		 (let* ((diff (car e))
+			(len (length e))
+			(lastx (e (- len 2)))
+			(newe (copy e)))
+		   (do ((i 0 (+ i 2)))
+		       ((>= i len) newe)
+		     (let ((x (/ (- (newe i) diff) lastx)))
+		       (set! xs (cons x xs))
+		       (set! (newe i) x))))))
+	      (remove-duplicates
+	       (lambda (lst)
+		 (let rem-dup ((lst lst)
+			       (nlst ()))
+		   (cond ((null? lst) nlst)
+			 ((member (car lst) nlst) (rem-dup (cdr lst) nlst))
+			 (else (rem-dup (cdr lst) (cons (car lst) nlst))))))))
 	  (if (null? e1)
 	      (at0 e2)
 	      (if (null? e2)
@@ -89,12 +85,11 @@ end: (window-envelope 1.0 3.0 '(0.0 0.0 5.0 1.0)) -> '(1.0 0.2 3.0 0.6)"))
 			(ee2 (at0 e2))
 			(newe ()))
 		    (set! xs (sort! (remove-duplicates xs) <))
-		    (let ((len (length xs)))
-		      (do ((i 0 (+ i 1)))
-			  ((= i len))
-			(let ((x (xs i)))
-			  (set! newe (append newe (list x (op (envelope-interp x ee1) (envelope-interp x ee2)))))))
-		      newe)))))))))
+		    (do ((len (length xs))
+			 (i 0 (+ i 1)))
+			((= i len) newe)
+		      (let ((x (xs i)))
+			(set! newe (append newe (list x (op (envelope-interp x ee1) (envelope-interp x ee2)))))))))))))))
 
 
 ;;; -------- multiply-envelopes, add-envelopes
@@ -116,36 +111,33 @@ envelope: (multiply-envelopes '(0 0 2 .5) '(0 0 1 2 2 1)) -> '(0 0 0.5 0.5 1.0 0
 (define max-envelope 
   (let ((documentation "(max-envelope env) -> max y value in env"))
     (lambda (env1)
-      (define (max-envelope-1 e mx)
-	(if (null? e)
-	    mx
-	    (max-envelope-1 (cddr e) (max mx (cadr e)))))
-      (max-envelope-1 (cddr env1) (cadr env1)))))
-
+      (let max-envelope-1 ((e (cddr env1))
+                           (mx (cadr env1)))
+        (if (null? e)
+            mx
+            (max-envelope-1 (cddr e) (max mx (cadr e))))))))
 
 ;;; -------- min-envelope
 
 (define min-envelope 
   (let ((documentation "(min-envelope env) -> min y value in env"))
     (lambda (env1)
-      (define (min-envelope-1 e mx)
-	(if (null? e)
-	    mx
-	    (min-envelope-1 (cddr e) (min mx (cadr e)))))
-      (min-envelope-1 (cddr env1) (cadr env1)))))
-
+      (let min-envelope-1 ((e (cddr env1))
+                           (mx (cadr env1)))
+        (if (null? e)
+            mx
+            (min-envelope-1 (cddr e) (min mx (cadr e))))))))
 
 ;;; -------- integrate-envelope
 
 (define integrate-envelope 
   (let ((documentation "(integrate-envelope env) -> area under env"))
     (lambda (env1)
-      (define (integrate-envelope-1 e sum)
-	(if (or (null? e) (null? (cddr e)))
-	    sum
-	    (integrate-envelope-1 (cddr e) (+ sum (* (+ (cadr e) (cadddr e)) .5 (- (caddr e) (car e)))))))
-      (integrate-envelope-1 env1 0.0))))
-
+      (let integrate-envelope-1 ((e env1)
+                                 (sum 0.0000))
+        (if (or (null? e) (null? (cddr e)))
+            sum
+            (integrate-envelope-1 (cddr e) (+ sum (* (+ (cadr e) (cadddr e)) 0.5 (- (caddr e) (car e))))))))))
 
 ;;; -------- envelope-last-x
 
@@ -170,68 +162,57 @@ divseg in early versions of CLM and its antecedents in Sambox and Mus10 (linen).
   (stretch-envelope '(0 0 1 1 2 0) .1 .2 1.5 1.6) -> (0 0 0.2 0.1 1.1 1 1.6 0.5 2.0 0)"))
     (lambda* (fn old-att new-att old-dec new-dec)
       
-      (if (and old-att
-	       (not new-att))
-	  (error 'wrong-number-of-args (list "stretch-envelope" 
-					     old-att
-					     "old-attack but no new-attack?"))
-	  (if (not new-att)
-	      fn
-	      (if (and old-dec
-		       (not new-dec))
-		  (error 'wrong-number-of-args (list "stretch-envelope" 
-						     old-att new-att old-dec
-						     "old-decay but no new-decay?"))
-		  (let* ((x0 (car fn))
-			 (new-x x0)
-			 (last-x (fn (- (length fn) 2)))
-			 (y0 (cadr fn))
-			 (new-fn (list y0 x0))
-			 (scl (/ (- new-att x0) (max .0001 (- old-att x0)))))
-		    (define (stretch-envelope-1 new-fn old-fn)
-		      (if (null? old-fn)
-			  new-fn
-			  (let ((x1 (car old-fn))
-				(y1 (cadr old-fn)))
-			    (if (and (< x0 old-att)
-				     (>= x1 old-att))
-				(begin
-				  (if (= x1 old-att)
-				      (set! y0 y1)
-				      (set! y0 (+ y0 (* (- y1 y0) (/ (- old-att x0) (- x1 x0))))))
-				  (set! x0 old-att)
-				  (set! new-x new-att)
-				  (set! new-fn (cons new-x new-fn))
-				  (set! new-fn (cons y0 new-fn))
-				  (set! scl (if old-dec 
-						(/ (- new-dec new-att) (- old-dec old-att))
-						(/ (- last-x new-att) (- last-x old-att))))))
-			    (if (and old-dec
+      (cond ((not new-att)
+	     (if old-att
+		 (error 'wrong-number-of-args "stretch-envelope: ~A, old-attack but no new-attack?" old-att)
+		 fn))
+	    ((and old-dec (not new-dec))
+	     (error 'wrong-number-of-args "stretch-envelope:~A ~A ~A, old-decay but no new-decay?" old-att new-att old-dec))
+	    (else
+	     (let ((x0 (car fn))
+		   (y0 (cadr fn)))
+	       (let ((new-x x0)
+		     (last-x (fn (- (length fn) 2)))
+		     (new-fn (list y0 x0))
+		     (scl (/ (- new-att x0) (max .0001 (- old-att x0)))))
+		 
+		 (if (and (number? old-dec)
+			  (= old-dec old-att))
+		     (set! old-dec (* 1e-06 last-x)))
+		 (reverse
+		  (let stretch-envelope-1 ((new-fn new-fn)
+					   (old-fn (cddr fn)))
+		    (if (null? old-fn)
+			new-fn
+			(let ((x1 (car old-fn))
+			      (y1 (cadr old-fn)))
+			  (when (and (< x0 old-att) (>= x1 old-att))
+			    (set! y0 (if (= x1 old-att)
+					 y1
+					 (+ y0 (* (- y1 y0) (/ (- old-att x0) (- x1 x0))))))
+			    (set! x0 old-att)
+			    (set! new-x new-att)
+			    (set! new-fn (cons y0 (cons new-x new-fn)))
+			    (set! scl (if old-dec
+					  (/ (- new-dec new-att) (- old-dec old-att))
+					  (/ (- last-x new-att) (- last-x old-att)))))
+			  (when (and (real? old-dec)
 				     (< x0 old-dec)
 				     (>= x1 old-dec))
-				(begin
-				  (if (= x1 old-dec)
-				      (set! y0 y1)
-				      (set! y0 (+ y0 (* (- y1 y0) (/ (- old-dec x0) (- x1 x0))))))
-				  (set! x0 old-dec)
-				  (set! new-x new-dec)
-				  (set! new-fn (cons new-x new-fn))
-				  (set! new-fn (cons y0 new-fn))
-				  (set! scl (/ (- last-x new-dec) (- last-x old-dec)))))
-			    (if (not (= x0 x1))
-				(begin
-				  (set! new-x (+ new-x (* scl (- x1 x0))))
-				  (set! new-fn (cons new-x new-fn))
-				  (set! new-fn (cons y1 new-fn))
-				  (set! x0 x1)
-				  (set! y0 y1)))
-			    (stretch-envelope-1 new-fn (cddr old-fn)))))
-		    
-		    (if (and old-dec 
-			     (= old-dec old-att)) 
-			(set! old-dec (* .000001 last-x)))
-		    (reverse (stretch-envelope-1 new-fn (cddr fn))))))))))
-
+			    (set! y0 (if (= x1 old-dec)
+					 y1
+					 (+ y0 (* (- y1 y0) (/ (- old-dec x0) (- x1 x0))))))
+			    (set! x0 old-dec)
+			    (set! new-x new-dec)
+			    (set! new-fn (cons y0 (cons new-x new-fn)))
+			    (set! scl (/ (- last-x new-dec) (- last-x old-dec))))
+			  (unless (= x0 x1)
+			    (set! new-x (+ new-x (* scl (- x1 x0))))
+			    (set! new-fn (cons y1 (cons new-x new-fn)))
+			    (set! x0 x1)
+			    (set! y0 y1))
+			  (stretch-envelope-1 new-fn (cddr old-fn)))))))))))))
+  
 
 ;;; -------- scale-envelope
 
@@ -240,8 +221,7 @@ divseg in early versions of CLM and its antecedents in Sambox and Mus10 (linen).
     (lambda* (e scl (offset 0))
       (if (null? e)
 	  ()
-	  (append (list (car e) (+ offset (* scl (cadr e))))
-		  (scale-envelope (cddr e) scl offset))))))
+	  (cons (car e) (cons (+ offset (* scl (cadr e))) (scale-envelope (cddr e) scl offset)))))))
 
 
 ;;; -------- reverse-envelope
@@ -258,10 +238,9 @@ divseg in early versions of CLM and its antecedents in Sambox and Mus10 (linen).
 				       newe))
 			   xd)))
       (let ((len (length e)))
-	(if (or (= len 0) (= len 2))
+	(if (memv len '(0 2))
 	    e
-	    (let ((xmax (e (- len 2))))
-	      (reverse-env-1 e () xmax)))))))
+	    (reverse-env-1 e () (e (- len 2))))))))
 
 
 ;;; -------- concatenate-envelopes
@@ -274,7 +253,7 @@ divseg in early versions of CLM and its antecedents in Sambox and Mus10 (linen).
 	    newe
 	    (cat-1 (cddr e)
 		   (cons (cadr e)
-			 (cons (+ (- (car e) x0) xoff)
+			 (cons (- (+ (car e) xoff) x0)
 			       newe))
 		   xoff
 		   x0)))
@@ -301,43 +280,41 @@ inserted between repeats. 'normalized' causes the new envelope's x axis
 to have the same extent as the original's. 'reflected' causes every other 
 repetition to be in reverse."))
     (lambda* (ur-env repeats reflected normalized)
-      (let* ((times (if reflected (floor (/ repeats 2)) repeats))
-	     (e (if reflected
-		    (let ((lastx (ur-env (- (length ur-env) 2)))
-			  (rev-env (cddr (reverse ur-env)))
-			  (new-env (reverse ur-env)))
-		      (while (pair? rev-env)
-			     (set! new-env (cons (+ lastx (- lastx (cadr rev-env))) new-env))
-			     (set! new-env (cons (car rev-env) new-env))
-			     (set! rev-env (cddr rev-env)))
-		      (reverse new-env))
-		    ur-env))
-	     (first-y (cadr e))
-	     (x-max (e (- (length e) 2)))
-	     (x (car e))
-	     (first-y-is-last-y (= first-y (e (- (length e) 1))))
-	     (new-env (list first-y x))
-	     (len (length e)))
-	(do ((i 0 (+ i 1)))
-	    ((= i times))
-	  (do ((j 2 (+ j 2)))
-	      ((>= j len))
-	    (set! x (+ x (- (e j) (e (- j 2)))))
-	    (set! new-env (cons x new-env))
-	    (set! new-env (cons (e (+ j 1)) new-env)))
-	  (if (and (< i (- times 1)) (not first-y-is-last-y))
-	      (begin
-		(set! x (+ x (/ x-max 100.0)))
-		(set! new-env (cons x new-env))
-		(set! new-env (cons first-y new-env)))))
-	(set! new-env (reverse new-env))
-	(if normalized
-	    (let ((scl (/ x-max x))
-		  (new-len (length new-env)))
-	      (do ((i 0 (+ i 2)))
-		  ((>= i new-len))
-		(set! (new-env i) (* scl (new-env i))))))
-	new-env))))
+      (let ((e (if (not reflected)
+		   ur-env
+		   (let ((lastx (ur-env (- (length ur-env) 2)))
+			 (rev-env (cddr (reverse ur-env)))
+			 (new-env (reverse ur-env)))
+		     (while (pair? rev-env)
+			    (set! new-env (cons (- (+ lastx lastx) (cadr rev-env)) new-env))
+			    (set! new-env (cons (car rev-env) new-env))
+			    (set! rev-env (cddr rev-env)))
+		     (reverse new-env)))))
+	(let ((first-y (cadr e))
+	      (x (car e)))
+	  (let ((x-max (e (- (length e) 2)))
+		(new-env (list first-y x)))
+	    (let ((len (length e))
+		  (times (if reflected (floor (/ repeats 2)) repeats))
+		  (first-y-is-last-y (= first-y (e (- (length e) 1)))))
+	      (do ((i 0 (+ i 1)))
+		  ((= i times))
+		(do ((j 2 (+ j 2)))
+		    ((>= j len))
+		  (set! x (- (+ x (e j)) (e (- j 2))))
+		  (set! new-env (cons (e (+ j 1)) (cons x new-env))))
+		(if (and (< i (- times 1)) (not first-y-is-last-y))
+		    (begin
+		      (set! x (+ x (/ x-max 100.0)))
+		      (set! new-env (cons first-y (cons x new-env)))))))
+	    (set! new-env (reverse new-env))
+	    (if normalized
+		(do ((scl (/ x-max x))
+		     (new-len (length new-env))
+		     (i 0 (+ i 2)))
+		    ((>= i new-len))
+		  (set! (new-env i) (* scl (new-env i)))))
+	    new-env))))))
 
 
 ;;; -------- power-env 
@@ -392,7 +369,7 @@ repetition to be in reverse."))
     (set! (pe 'current-pass) (- (length (vector-ref (pe 'envs) 0)) 1))
     pe))
 
-(define* (power-env-channel pe (beg 0) dur snd chn edpos (edname "power-env-channel"))
+(define* (power-env-channel pe (beg 0) snd chn edpos (edname "power-env-channel"))
   ;; split into successive calls on env-channel
   (let ((curbeg beg)) ; sample number
     (as-one-edit
@@ -412,30 +389,30 @@ repetition to be in reverse."))
   (let ((documentation "(powenv-channel envelope (beg 0) dur snd chn edpos) returns an envelope with a separate base for \
 each segment: (powenv-channel '(0 0 .325  1 1 32.0 2 0 32.0))"))
     (lambda* (envelope (beg 0) dur snd chn edpos)
-      (let* ((curbeg beg)
-	     (fulldur (or dur (framples snd chn edpos)))
-	     (len (length envelope))
-	     (x1 (car envelope))
-	     (xrange (- (envelope (- len 3)) x1))
-	     (y1 (cadr envelope))
-	     (base (caddr envelope))
-	     (x0 0.0)
-	     (y0 0.0))
-	(if (= len 3)
-	    (scale-channel y1 beg dur snd chn edpos)
-	    (as-one-edit
-	     (lambda ()
-	       (do ((i 3 (+ i 3)))
-		   ((= i len))
-		 (set! x0 x1)
-		 (set! y0 y1)
-		 (set! x1 (envelope i))
-		 (set! y1 (envelope (+ i 1)))
-		 (let ((curdur (round (* fulldur (/ (- x1 x0) xrange)))))
-		   (xramp-channel y0 y1 base curbeg curdur snd chn edpos)
-		   (set! curbeg (+ curbeg curdur)))
-		 (set! base (envelope (+ i 2)))))))))))
-
+      (let ((len (length envelope))
+	    (x1 (car envelope)))
+	(let ((curbeg beg)
+	      (fulldur (or dur (framples snd chn edpos)))
+	      (xrange (- (envelope (- len 3)) x1))
+	      (y1 (cadr envelope))
+	      (base (caddr envelope))
+	      (x0 0.0)
+	      (y0 0.0))
+	  (if (= len 3)
+	      (scale-channel y1 beg dur snd chn edpos)
+	      (as-one-edit
+	       (lambda ()
+		 (do ((i 3 (+ i 3)))
+		     ((= i len))
+		   (set! x0 x1)
+		   (set! y0 y1)
+		   (set! x1 (envelope i))
+		   (set! y1 (envelope (+ i 1)))
+		   (let ((curdur (round (* fulldur (/ (- x1 x0) xrange)))))
+		     (xramp-channel y0 y1 base curbeg curdur snd chn edpos)
+		     (set! curbeg (+ curbeg curdur)))
+		   (set! base (envelope (+ i 2))))))))))))
+  
 
 ;;; by Anders Vinjar:
 ;;;
@@ -451,25 +428,22 @@ each segment: (powenv-channel '(0 0 .325  1 1 32.0 2 0 32.0))"))
 (define envelope-exp 
   (let ((documentation "(envelope-exp e (power 1.0) (xgrid 100)) approximates an exponential curve connecting the breakpoints"))
     (lambda* (e (power 1.0) (xgrid 100))
-      (let* ((mn (min-envelope e))
-	     (largest-diff (* 1.0 (- (max-envelope e) mn)))
-	     (x-min (car e))
-	     (len (length e))
-	     (x-max (e (- len 2)))
-	     (x-incr (* 1.0 (/ (- x-max x-min) xgrid)))
-	     (new-e ()))
-	(do ((x x-min (+ x x-incr)))
-	    ((>= x x-max))
-	  (let ((y (envelope-interp x e)))
-	    (set! new-e (cons x new-e))
-	    (set! new-e (cons (if (= largest-diff 0.0)
-				  y
-				  (+ mn
-				     (* largest-diff
-					(expt (/ (- y mn) largest-diff) power))))
-			      new-e))))
-	(reverse new-e)))))
-
+      (let ((mn (min-envelope e)))
+	(let ((largest-diff (* 1.0 (- (max-envelope e) mn)))
+	      (x-min (car e))
+	      (x-max (e (- (length e) 2))))
+	  (do ((x-incr (* 1.0 (/ (- x-max x-min) xgrid)))
+	       (new-e ())
+	       (x x-min (+ x x-incr)))
+	      ((>= x x-max)
+	       (reverse new-e))     
+	    (let ((y (envelope-interp x e)))
+	      (set! new-e (cons (if (= largest-diff 0.0)
+				    y
+				    (+ mn
+				       (* largest-diff
+					  (expt (/ (- y mn) largest-diff) power))))
+				(cons x new-e))))))))))
 
 ;;; rms-envelope
 
@@ -477,47 +451,46 @@ each segment: (powenv-channel '(0 0 .325  1 1 32.0 2 0 32.0))"))
   (let ((documentation "(rms-envelope file (beg 0.0) (dur #f) (rfreq 30.0) (db #f)) returns an envelope of RMS values in 'file'"))
     (lambda* (file (beg 0.0) dur (rfreq 30.0) db)
       ;; based on rmsenv.ins by Bret Battey
-      (let* ((e ())
-	     (incr (/ 1.0 rfreq))
-	     (fsr (srate file))
-	     (incrsamps (round (* incr fsr)))
+      (let* ((fsr (srate file))
 	     (start (round (* beg fsr)))
-	     (reader (make-sampler start file))
 	     (end (if dur (min (* 1.0 (+ start (round (* fsr dur))))
 			       (mus-sound-framples file))
-		      (mus-sound-framples file)))
-	     (rms (make-moving-average incrsamps)) ; this could use make-moving-rms from dsp.scm
-	     (rms-val 0.0)
-	     (jend 0))
-	(let* ((len (+ 1 (- end start)))
-	       (data (make-float-vector len)))
-	  (do ((i 0 (+ i 1)))
-	      ((= i len))
-	    (float-vector-set! data i (next-sample reader)))
-	  (float-vector-multiply! data data)
-	  (do ((i 0 (+ i incrsamps)))
-	      ((>= i end) 
-	       (reverse e))
-	    (set! jend (min end (+ i incrsamps)))
-	    (do ((j i (+ j 1)))
-		((= j jend))
-	      (moving-average rms (float-vector-ref data j)))
-	    (set! e (cons (* 1.0 (/ i fsr)) e))
-	    (set! rms-val (sqrt (* (mus-scaler rms) (mus-increment rms))))
-	    (if db 
-		(if (< rms-val .00001)
-		    (set! e (cons -100.0 e))
-		    (set! e (cons (* 20.0 (log rms-val 10.0)) e)))
-		(set! e (cons rms-val e)))))))))
-
-
+		      (mus-sound-framples file))))
+	(let ((incrsamps (round (/ fsr rfreq)))
+	      (len (- (+ end 1) start)))
+	  (let ((reader (make-sampler start file))
+		(rms (make-moving-average incrsamps)) ; this could use make-moving-rms from dsp.scm
+		(e ())
+		(rms-val 0.0)
+		(jend 0)
+		(data (make-float-vector len)))
+	    (do ((i 0 (+ i 1)))
+		((= i len))
+	      (float-vector-set! data i (next-sample reader)))
+	    (float-vector-multiply! data data)
+	    (do ((i 0 (+ i incrsamps)))
+		((>= i end) 
+		 (reverse e))
+	      (set! jend (min end (+ i incrsamps)))
+	      (do ((j i (+ j 1)))
+		  ((= j jend))
+		(moving-average rms (float-vector-ref data j)))
+	      (set! e (cons (* 1.0 (/ i fsr)) e))
+	      (set! rms-val (sqrt (* (mus-scaler rms) (mus-increment rms))))
+	      (set! e (cons (if db 
+				(if (< rms-val 1e-05) -100.0 (* 20.0 (log rms-val 10.0)))
+				rms-val)
+			    e)))))))))
+  
+  
 (define* (normalize-envelope env1 (new-max 1.0))
-  (define (abs-max-envelope-1 e mx)
-    (if (null? e)
-	mx
-	(abs-max-envelope-1 (cddr e) (max mx (abs (cadr e))))))
-  (let ((peak (abs-max-envelope-1 (cddr env1) (abs (cadr env1)))))
-    (scale-envelope env1 (/ new-max peak))))
+  (scale-envelope env1
+		  (/ new-max
+		     (let abs-max-envelope-1 ((e (cddr env1))
+					      (mx (abs (cadr env1))))
+		       (if (null? e)
+			   mx
+			   (abs-max-envelope-1 (cddr e) (max mx (abs (cadr e)))))))))
 
 
 ;;; simplify-envelope
@@ -540,27 +513,18 @@ each segment: (powenv-channel '(0 0 .325  1 1 32.0 2 0 32.0))"))
     (if (or (= py qy ty) 
 	    (= px qx tx))
 	:within
-	(if (< (abs (- (* (- qy py) (- tx px))
-		       (* (- ty py) (- qx px))))
-	       (max (abs (- qx px))
-		    (abs (- qy py))))
-	    (if (or (and (< qx px) (< px tx))
-		    (and (< qy py) (< py ty)))
-		:before
-		(if (or (and (< tx px) (< px qx))
-			(and (< ty py) (< py qy)))
-		    :before
-		    (if (or (and (< px qx) (< qx tx))
-			    (and (< py qy) (< qy ty)))
-			:after
-			(if (or (and (< tx qx) (< qx px))
-				(and (< ty qy) (< qy py)))
-			    :after
-			    :within))))
-	    #f)))
-  
-  (if (and env1
-	   (> (length env1) 4))
+	(and (< (abs (- (* (- qy py) (- tx px))
+			(* (- ty py) (- qx px))))
+		(max (abs (- qx px))
+		     (abs (- qy py))))
+	     (if (or (< qx px tx) (< qy py ty) (< tx px qx) (< ty py qy)) 
+		 :before
+		 (if (or (< px qx tx) (< py qy ty) (< tx qx px) (< ty qy py)) 
+		     :after
+		     :within)))))
+  (if (not (and env1
+		(> (length env1) 4)))
+      env1
       (let ((new-env (list (cadr env1) (car env1)))
 	    (ymax (max-envelope env1))
 	    (ymin (min-envelope env1))
@@ -568,33 +532,31 @@ each segment: (powenv-channel '(0 0 .325  1 1 32.0 2 0 32.0))"))
 	    (xmin (car env1)))
 	(if (= ymin ymax)
 	    (list xmin ymin xmax ymax)
-	    (let ((y-scl (/ ygrid (- ymax ymin)))
-		  (x-scl (/ (or xgrid ygrid) (- xmax xmin)))
-		  (px #f) (py #f)
-		  (qx #f) (qy #f) 
-		  (tx #f) (ty #f) 
-		  (qtx #f) (qty #f))
-	      (do ((i 0 (+ i 2)))
-		  ((>= i (length env1)))
-		(let ((ttx (env1 i))
-		      (tty (env1 (+ i 1))))
-		  (set! tx (round (* ttx x-scl)))
-		  (set! ty (round (* tty y-scl)))
-		  (if px
-		      (if (not (point-on-line? px py qx qy tx ty))
-			  (begin
-			    (set! new-env (cons qtx new-env))
-			    (set! new-env (cons qty new-env))
-			    (set! px qx)
-			    (set! py qy)))
-		      (begin
-			(set! px qx)
-			(set! py qy)))
-		  (set! qx tx)
-		  (set! qy ty)
-		  (set! qtx ttx)
-		  (set! qty tty)))
-	      (set! new-env (cons qtx new-env))
-	      (set! new-env (cons qty new-env))
-	      (reverse new-env))))
-      env1))
+	    (do ((y-scl (/ ygrid (- ymax ymin)))
+		 (x-scl (/ (or xgrid ygrid) (- xmax xmin)))
+		 (px #f) (py #f)
+		 (qx #f) (qy #f) 
+		 (tx #f) (ty #f) 
+		 (qtx #f) (qty #f)
+		 (i 0 (+ i 2)))
+		((>= i (length env1))
+		 (set! new-env (cons qty (cons qtx new-env)))
+		 (reverse new-env))
+	      (let ((ttx (env1 i))
+		    (tty (env1 (+ i 1))))
+		(set! tx (round (* ttx x-scl)))
+		(set! ty (round (* tty y-scl)))
+		(if px
+		    (if (not (point-on-line? px py qx qy tx ty))
+			(begin
+			  (set! new-env (cons qty (cons qtx new-env)))
+			  (set! px qx)
+			  (set! py qy)))
+		    (begin
+		      (set! px qx)
+		      (set! py qy)))
+		(set! qx tx)
+		(set! qy ty)
+		(set! qtx ttx)
+		(set! qty tty)))))))
+	    
\ No newline at end of file
