@@ -43,46 +43,46 @@
   (let ((snd (hook 'snd))
 	(chn (hook 'chn))
 	(ux (hook 'x))
-	(uy (hook 'y))
-	(mouse-radius .03))
+	(uy (hook 'y)))
     
     (define (add-envelope-point x y cur-env)
       (let ((new-env ()))
-	(define (search-point e)
-	  (cond ((null? e)     (append new-env (list x y)))
+	(let search-point ((e cur-env))
+	  (cond ((null? e) (append new-env (list x y)))
 		((= (car e) x) (append new-env (list x y) (cddr e)))
 		((> (car e) x) (append new-env (list x y) e))
 		(else
 		 (set! new-env (append new-env (list (car e) (cadr e))))
-		 (search-point (cddr e)))))
-	(search-point cur-env)))
-    
+		 (search-point (cddr e)))))))
+      
     (define (envelope-position x cur-env)
-      (define (search-point e pos)
-	(if (= (car e) x)
-	    pos
-	    (search-point (cddr e) (+ pos 2))))
-      (search-point cur-env 0))
+      (let search-point ((e cur-env)
+                         (pos 0))
+        (if (= (car e) x)
+            pos
+            (search-point (cddr e) (+ pos 2)))))
     
-    (define (on-dot? x y cur-env pos)
-      (and (pair? cur-env)
-	   (or (and (< (abs (- (car cur-env) x)) mouse-radius)
-		    (< (abs (- (cadr cur-env) y)) mouse-radius)
-		    pos)
-	       (on-dot? x y (cddr cur-env) (+ pos 2)))))
-    
-    (let* ((x (max 0.0 (min ux 1.0)))
-	   (y (max 0.0 (min uy 1.0)))
-	   (cur-env (channel-envelope snd chn))
-	   (pos (on-dot? x y cur-env 0)))
-      (set! mouse-new (not pos))
-      (set! mouse-down (get-internal-real-time))
-      (if (not pos)
-	  (let ((new-x (max 0.001 (min x .999))))
-	    (set! (channel-envelope snd chn) 
-		  (add-envelope-point new-x y cur-env))
-	    (set! mouse-pos (envelope-position new-x (channel-envelope snd chn))))
-	  (set! mouse-pos pos)))))
+    (letrec ((on-dot? (let ((mouse-radius .03))
+			(lambda (x y cur-env pos)
+			  (and (pair? cur-env)
+			       (pair? (cdr cur-env))
+			       (or (and (< (abs (- (car cur-env) x)) mouse-radius)
+					(< (abs (- (cadr cur-env) y)) mouse-radius)
+					pos)
+				   (on-dot? x y (cddr cur-env) (+ pos 2))))))))
+      (let* ((x (max 0.0 (min ux 1.0)))
+	     (y (max 0.0 (min uy 1.0)))
+	     (cur-env (channel-envelope snd chn))
+	     (pos (on-dot? x y cur-env 0)))
+	(set! mouse-new (not pos))
+	(set! mouse-down (get-internal-real-time))
+	(if pos
+	    (set! mouse-pos pos)
+	    (let ((new-x (max 0.001 (min x .999))))
+	      (set! (channel-envelope snd chn) 
+		    (add-envelope-point new-x y cur-env))
+	      (set! mouse-pos (envelope-position new-x (channel-envelope snd chn)))))))))
+
 
 (define (mouse-drag-envelope hook)
   (let ((snd (hook 'snd))
@@ -93,26 +93,26 @@
     
     (define (edit-envelope-point pos x y cur-env)
       (let ((new-env ()))
-	(define (search-point e npos)
+	(let search-point ((e cur-env)
+			   (npos 0))
 	  (if (= npos pos)
 	      (append new-env (list x y) (cddr e))
 	      (begin
 		(set! new-env (append new-env (list (car e) (cadr e))))
-		(search-point (cddr e) (+ npos 2)))))
-	(search-point cur-env 0)))
-    
-    (let* ((cur-env (channel-envelope snd chn))
-	   (lx (if (= mouse-pos 0)
-		   0.0
-		   (if (>= mouse-pos (- (length cur-env) 2))
-		       1.0
-		       (max (+ (list-ref cur-env (- mouse-pos 2)) .001)
-			    (min x
-				 (- (list-ref cur-env (+ mouse-pos 2)) .001))))))
-	   (ly (max 0.0 (min y 1.0))))
-      (set! (channel-envelope snd chn) 
-	    (edit-envelope-point mouse-pos lx ly cur-env))
-      (update-lisp-graph snd chn))))
+		(search-point (cddr e) (+ npos 2)))))))
+
+    (let ((cur-env (channel-envelope snd chn)))
+      (let ((lx (if (= mouse-pos 0)
+		    0.0
+		    (if (>= mouse-pos (- (length cur-env) 2))
+			1.0
+			(max (+ (list-ref cur-env (- mouse-pos 2)) .001)
+			     (min x
+				  (- (list-ref cur-env (+ mouse-pos 2)) .001))))))
+	    (ly (max 0.0 (min y 1.0))))
+	(set! (channel-envelope snd chn) 
+	      (edit-envelope-point mouse-pos lx ly cur-env))
+	(update-lisp-graph snd chn)))))
 
 (define (mouse-release-envelope hook)
   (let ((snd (hook 'snd))
@@ -121,44 +121,44 @@
     
     (define (remove-envelope-point pos cur-env)
       (let ((new-env ()))
-	(define (search-point e npos)
+	(let search-point ((e cur-env)
+			   (npos 0))
 	  (if (null? e)
 	      new-env
 	      (if (= pos npos)
 		  (append new-env (cddr e))
 		  (begin
 		    (set! new-env (append new-env (list (car e) (cadr e))))
-		    (search-point (cddr e) (+ npos 2))))))
-	(search-point cur-env 0)))
+		    (search-point (cddr e) (+ npos 2))))))))
     
-    (if (= axis lisp-graph)
-	(let ((cur-env (channel-envelope snd chn)))
-	  (set! mouse-up (get-internal-real-time))
-	  (if (and (not mouse-new)
-		   (<= (- mouse-up mouse-down) click-time)
-		   (not (= mouse-pos 0))
-		   (< mouse-pos (- (length cur-env) 2)))
-	      (set! (channel-envelope snd chn)
-		    (remove-envelope-point mouse-pos cur-env)))
-	  (update-lisp-graph snd chn)
-	  (set! mouse-new #f)
-	  (set! (hook 'result) #t)))))
+    (when (= axis lisp-graph)
+      (let ((cur-env (channel-envelope snd chn)))
+	(set! mouse-up (get-internal-real-time))
+	(if (and (not mouse-new)
+		 (<= (- mouse-up mouse-down) click-time)
+		 (not (= mouse-pos 0))
+		 (< mouse-pos (- (length cur-env) 2)))
+	    (set! (channel-envelope snd chn)
+		  (remove-envelope-point mouse-pos cur-env))))
+      (update-lisp-graph snd chn)
+      (set! mouse-new #f)
+      (set! (hook 'result) #t))))
 
 (define (enveloping-key-press hook)
   (let ((snd (hook 'snd))
 	(chn (hook 'chn))
 	(key (hook 'key))
-	(state (hook 'state)))
+	(state4 (= (hook 'state) 4)))
     
     ;; C-g returns to original env
     ;; C-. applies current env to amplitude
     (if (and (= key (char->integer #\.))
-	     (= state 4))
+	     state4)
 	(begin
 	  (env-channel (channel-envelope snd chn) 0 (framples snd chn) snd chn)
 	  (set! (hook 'result) #t))
 	(if (and (= key (char->integer #\g))
-		 (= state 4))
+		 state4)
 	    (begin
 	      (set! (channel-envelope snd chn) '(0.0 1.0 1.0 1.0))
 	      (set! (hook 'result) #t))))))
@@ -189,17 +189,17 @@
 (define play-with-envs
   (let ((documentation "(play-with-envs snd) sets channel amps during playback from the associated enved envelopes"))
     (lambda* (sound)
-      (let ((chans (channels sound)))
-	(do ((chan 0 (+ 1 chan)))
-	    ((= chan chans))
-	  (let ((player (make-player sound chan))
-		(e (make-env (channel-envelope sound chan) 
-			     :length (floor (/ (framples sound chan) *dac-size*)))))
-	    (add-player player 0 -1 -1 (lambda (reason) (set! (hook-functions play-hook) ())))
-	    (hook-push play-hook (lambda (hook)
-				   ;; if dac buffer size in framples is not dac-size, we should do something debonair
-				   (set! (amp-control player) (env e))))))
-	(start-playing chans (srate sound))))))
+      (do ((chans (channels sound))
+	   (chan 0 (+ 1 chan)))
+	  ((= chan chans)
+	   (start-playing chans (srate sound)))   
+	(let ((player (make-player sound chan))
+	      (e (make-env (channel-envelope sound chan) 
+			   :length (floor (/ (framples sound chan) *dac-size*)))))
+	  (add-player player 0 -1 -1 (lambda (reason) (set! (hook-functions play-hook) ())))
+	  (hook-push play-hook (lambda (hook)
+				 ;; if dac buffer size in framples is not dac-size, we should do something debonair
+				 (set! (amp-control player) (env e)))))))))
 
 #|
 (define play-panned 
