@@ -61,7 +61,7 @@
   
   (define (display-scanned-synthesis)
     
-    (define compute-uniform-circular-string
+    (define vibrating-uniform-circular-string
       ;; copied from dsp.scm to simplify life
       (lambda (size x0 x1 x2 mass xspring damp)
 	(define circle-float-vector-ref 
@@ -193,7 +193,7 @@
       
       (define (tick-synthesis n)
 	;; background process
-	(compute-uniform-circular-string size gx0 gx1 gx2 mass xspring damp)
+	(vibrating-uniform-circular-string size gx0 gx1 gx2 mass xspring damp)
 	(let* ((wn (GDK_WINDOW (gtk_widget_get_window scan-pane)))
 	       (cr (gdk_cairo_create wn)))
 	  (draw-graph cr)
@@ -350,16 +350,14 @@
 	      (snd (car context))
 	      (chn (cadr context)))
 	  (if (sound-property 'dragger snd)
-	      (begin
-		(do ((i 0 (+ i 1)))
-		    ((= i (channels snd)))
-		  (if (not (= i chn))
-		      (begin
-			(set! (y-zoom-slider snd i) (* val val))
-			(set! (y-position-slider snd i) (y-position-slider snd chn)))))
-		(g_signal_stop_emission (GPOINTER adj)
-					(g_signal_lookup "value_changed" (G_OBJECT_TYPE (G_OBJECT adj)))
-					0)))))
+	      (do ((i 0 (+ i 1)))
+		  ((= i (channels snd))
+		   (g_signal_stop_emission (GPOINTER adj)
+					   (g_signal_lookup "value_changed" (G_OBJECT_TYPE (G_OBJECT adj)))
+					   0))
+		(unless (= i chn)
+		  (set! (y-zoom-slider snd i) (* val val))
+		  (set! (y-position-slider snd i) (y-position-slider snd chn)))))))
       
       (set! (sound-property 'dragger snd) #t)
       (set! (sound-property 'save-state-ignore snd)
@@ -413,13 +411,10 @@
     (let ((labelled-snds ()))
       
       (define (kmg num)
-	(if (<= num 0)
-	    "disk full!"
-	    (if (> num 1024)
-		(if (> num (* 1024 1024))
-		    (format #f "space: ~6,3FG" (/ num (* 1024.0 1024.0)))
-		    (format #f "space: ~6,3FM" (/ num 1024.0)))
-		(format #f "space: ~10DK" num))))
+	(cond ((<= num 0)      (copy "disk full!"))
+	      ((<= num 1024)   (format #f "space: ~10DK" num))
+	      ((> num 1048576) (format #f "space: ~6,3FG" (/ num (* 1024.0 1024.0))))
+	      (else            (format #f "space: ~6,3FM" (/ num 1024.0)))))
       
       (define (show-label data)
 	(if (sound? (car data))
@@ -428,10 +423,10 @@
 	      (g_timeout_add 10000 show-label data) ; every 10 seconds recheck space
 	      0)))
       
-      (define (find-if pred l)
-	(cond ((null? l) #f)
-	      ((pred (car l)) (car l))
-	      (else (find-if pred (cdr l)))))
+      (define (find-if pred lst)
+	(cond ((null? lst) #f)
+	      ((pred (car lst)) (car lst))
+	      (else (find-if pred (cdr lst)))))
       
       (lambda (hook)
 	;; (show-disk-space snd) adds a label to snd's status-area area showing the current free space (for use with after-open-hook)
@@ -442,8 +437,7 @@
 	      (if (not snd)
 		  (snd-error "no sound found for disk space label")
 		  (let* ((name-form ((sound-widgets) 10))
-			 (space (kmg (disk-kspace (file-name snd))))
-			 (new-label (gtk_label_new space)))
+			 (new-label (gtk_label_new (kmg (disk-kspace (file-name snd))))))
 		    (gtk_box_pack_start (GTK_BOX name-form) new-label #f #f 6)
 		    (gtk_widget_show new-label)
 		    (set! previous-label (list snd new-label))
@@ -479,10 +473,9 @@
   
   (define snd-clock-icon
     (lambda (snd hour)
-      (let* ((window (GDK_WINDOW (gtk_widget_get_window ((sound-widgets snd) 8))))
-	     (cr (gdk_cairo_create window))
-	     (bg (color->list *basic-color*)))
-	(cairo_set_source_rgb cr (car bg) (cadr bg) (caddr bg))
+      (let ((cr (gdk_cairo_create (GDK_WINDOW (gtk_widget_get_window ((sound-widgets snd) 8))))))
+	(let ((bg (color->list *basic-color*)))
+	  (cairo_set_source_rgb cr (car bg) (cadr bg) (caddr bg)))
 	(cairo_rectangle cr 0 0 16 16) ; icon bg
 	(cairo_fill cr)
 	(cairo_set_source_rgb cr 1.0 1.0 1.0)
@@ -503,11 +496,10 @@
   (define (snd-happy-face snd progress)
     (let* ((window (GDK_WINDOW (gtk_widget_get_window ((sound-widgets snd) 8))))
 	   (cr (gdk_cairo_create window))
-	   (bg (color->list *basic-color*))
 	   (fc (list 1.0 progress 0.0)))
-      
-      ;; overall background
-      (cairo_set_source_rgb cr (car bg) (cadr bg) (caddr bg))
+      (let ((bg (color->list *basic-color*)))
+        ;; overall background
+        (cairo_set_source_rgb cr (car bg) (cadr bg) (caddr bg)))
       (cairo_rectangle cr 0 0 16 16)
       (cairo_fill cr)
       
@@ -564,17 +556,19 @@
       ;; (list (list widget inuse func title help) ...)
       
       (define (find-free-dialog ds)
-	(if (null? ds)
-	    #f
-	    (if (not (cadr (car ds)))
-		(begin
-		  (set! ((car ds) 1) #t)
-		  (caar ds))
-		(find-free-dialog (cdr ds)))))
+	(and (pair? ds)
+	     (pair? (car ds))
+	     (pair? (cdar ds))
+	     (if (cadar ds)
+		 (find-free-dialog (cdr ds))
+		 (begin
+		   (set! ((car ds) 1) #t)
+		   (caar ds)))))
+
       (lambda args
 	;; (file-select func title dir filter help)
-	(let* ((func (if (> (length args) 0) (args 0) #f))
-	       (title (if (> (length args) 1) (args 1) "select file"))
+	(let* ((func (and (pair? args) (args 0)))
+	       (title (if (and (pair? args) (pair? (cdr args))) (args 1) "select file"))
 	       (dir (if (> (length args) 2) (args 2) "."))
 	       (dialog (or (find-free-dialog file-selector-dialogs)
 			   (GTK_FILE_CHOOSER_DIALOG (gtk_file_chooser_dialog_new
@@ -766,26 +760,25 @@
   (define variables-pages ())
   
   (define (make-variables-dialog)
-    (let ((dismiss-button #f))
-      (set! variables-dialog (gtk_dialog_new))
-      (gtk_window_set_title (GTK_WINDOW variables-dialog) "Variables")
-      (gtk_container_set_border_width (GTK_CONTAINER variables-dialog) 10)
-      (gtk_window_set_default_size (GTK_WINDOW variables-dialog) -1 -1)
-      (gtk_window_set_resizable (GTK_WINDOW variables-dialog) #t)
-      (gtk_widget_realize variables-dialog)
-      (g_signal_connect variables-dialog "delete_event" (lambda (w ev data) (gtk_widget_hide variables-dialog) #t) #f)
+    (set! variables-dialog (gtk_dialog_new))
+    (gtk_window_set_title (GTK_WINDOW variables-dialog) "Variables")
+    (gtk_container_set_border_width (GTK_CONTAINER variables-dialog) 10)
+    (gtk_window_set_default_size (GTK_WINDOW variables-dialog) -1 -1)
+    (gtk_window_set_resizable (GTK_WINDOW variables-dialog) #t)
+    (gtk_widget_realize variables-dialog)
+    (g_signal_connect variables-dialog "delete_event" (lambda (w ev data) (gtk_widget_hide variables-dialog) #t) #f)
       
-      (set! dismiss-button (gtk_dialog_add_button (GTK_DIALOG variables-dialog) "Go Away" GTK_RESPONSE_NONE))
+    (let ((dismiss-button (gtk_dialog_add_button (GTK_DIALOG variables-dialog) "Go Away" GTK_RESPONSE_NONE)))
       (g_signal_connect dismiss-button "clicked" (lambda (w data) (gtk_widget_hide variables-dialog)) #f)
       (gtk_widget_show dismiss-button)
-      (gtk_widget_set_name dismiss-button "quit_button")
+      (gtk_widget_set_name dismiss-button "quit_button"))
       
-      (set! variables-notebook (gtk_notebook_new))
-      (gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG variables-dialog))) variables-notebook #t #t 4)
-      (gtk_notebook_set_tab_pos (GTK_NOTEBOOK variables-notebook) GTK_POS_RIGHT)
-      (gtk_widget_show variables-notebook)
-      (gtk_widget_show variables-dialog)
-      variables-dialog))
+    (set! variables-notebook (gtk_notebook_new))
+    (gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG variables-dialog))) variables-notebook #t #t 4)
+    (gtk_notebook_set_tab_pos (GTK_NOTEBOOK variables-notebook) GTK_POS_RIGHT)
+    (gtk_widget_show variables-notebook)
+    (gtk_widget_show variables-dialog)
+    variables-dialog)
   
   (define* (make-variable-display page-name variable-name (type 'text) (range (list 0.0 1.0)))
     ;; type = 'text, 'meter, 'graph, 'spectrum, 'scale
@@ -804,16 +797,15 @@
 	(case type
 	  ((text)
 	   ;; add a horizontal pair: label text
-	   (let ((label (gtk_label_new var-label))
-		 (hbox (gtk_box_new GTK_ORIENTATION_HORIZONTAL 0))
-		 (text (gtk_label_new "")))
-	     (gtk_box_pack_start (GTK_BOX pane) hbox #f #f 2)
-	     (gtk_widget_show hbox)
-	     (gtk_box_pack_start (GTK_BOX hbox) label #f #f 6)
-	     
-	     (gtk_widget_set_halign (GTK_WIDGET label) GTK_ALIGN_START)
-	     (gtk_widget_show label)
-	     (gtk_box_pack_start (GTK_BOX hbox) text #t #t 6)
+	   (let ((text (gtk_label_new "")))
+	     (let ((hbox (gtk_box_new GTK_ORIENTATION_HORIZONTAL 0)))
+	       (let ((label (gtk_label_new var-label)))
+		 (gtk_box_pack_start (GTK_BOX pane) hbox #f #f 2)
+		 (gtk_widget_show hbox)
+		 (gtk_box_pack_start (GTK_BOX hbox) label #f #f 6)
+		 (gtk_widget_set_halign (GTK_WIDGET label) GTK_ALIGN_START)
+		 (gtk_widget_show label))
+	       (gtk_box_pack_start (GTK_BOX hbox) text #t #t 6))
 	     (gtk_widget_set_halign (GTK_WIDGET text) GTK_ALIGN_START)
 	     (gtk_widget_show text)
 	     text))
@@ -846,7 +838,7 @@
       (gdk_window_invalidate_rect (GDK_WINDOW (gtk_widget_get_window (GTK_WIDGET wid))) (list 'GdkRectangle_ 0) #t)
       (gdk_window_process_updates (GDK_WINDOW (gtk_widget_get_window (GTK_WIDGET wid))) #t))
     
-    (define (widget? w) (and (list? w) (= (length w) 2) (eq? (car w) 'GtkWidget_)))
+    (define (widget? w) (and (pair? w) (= (length w) 2) (eq? (car w) 'GtkWidget_)))
     
     ;; (let ((wid1 (make-variable-display "do-loop" "i*1" 'spectrum))) (variable-display 0.1 wid1))
     
@@ -856,29 +848,26 @@
 	      (gtk_label_set_text (GTK_LABEL widget) (object->string var))
 	      (force-update widget)))
 	
-	(if (list? widget)
-	    (if (or (number? (car widget))
-		    (sound? (car widget)))
-		;; graph/spectrum -- does this need an explicit update?
-		(let* ((snd (car widget))
-		       (data (cadr widget))
-		       (len (length data))
-		       (loc (cursor snd 0)))
-		  (set! (data loc) var)
-		  (if (time-graph? snd) (update-time-graph snd))
-		  (if (transform-graph? snd) (update-transform-graph snd))
-		  (if (= (+ loc 1) len)
-		      (set! (cursor snd 0) 0)
-		      (set! (cursor snd 0) (+ loc 1))))
-		(if (GTK_IS_PROGRESS_BAR (car widget))
-		    ;; "thermometer"
-		    (let ((y0 (cadr widget))
-			  (y1 (caddr widget)))
-		      ;; (define wid (make-variable-display "do-loop" "i*2" 'scale))
-		      (gtk_progress_bar_set_fraction 
-		       (GTK_PROGRESS_BAR (car widget))
-		       (max 0.0 (min 1.0 (/ (- var y0) (- y1 y0))))))
-		    ))))
+	(when (and (pair? widget)
+		   (or (number? (car widget))
+		       (sound? (car widget))))
+	  ;; graph/spectrum -- does this need an explicit update?
+	  (let ((snd (car widget))
+		(data (cadr widget)))
+	    (let ((len (length data))
+		  (loc (cursor snd 0)))
+	      (set! (data loc) var)
+	      (if (time-graph? snd) (update-time-graph snd))
+	      (if (transform-graph? snd) (update-transform-graph snd))
+	      (set! (cursor snd 0) (if (= (+ loc 1) len) 0 (+ loc 1))))
+	    (if (GTK_IS_PROGRESS_BAR (car widget))
+		;; "thermometer"
+		(let ((y0 (cadr widget))
+		      (y1 (caddr widget)))
+		  ;; (define wid (make-variable-display "do-loop" "i*2" 'scale))
+		  (gtk_progress_bar_set_fraction 
+		   (GTK_PROGRESS_BAR (car widget))
+		   (max 0.0 (min 1.0 (/ (- var y0) (- y1 y0))))))))))
     var)
 
   (define (notebook-with-top-tabs)
