@@ -48,9 +48,7 @@
 		   (for-each
 		    (lambda (a)
 		      (if (not (keyword? a))
-			  (if (symbol? a)
-			      (set! arg-names (cons a arg-names))
-			      (set! arg-names (cons (car a) arg-names)))))
+			  (set! arg-names (cons (if (symbol? a) a (car a)) arg-names))))
 		    targs)
 		   (reverse arg-names))))
     (if (string? (car body))
@@ -122,12 +120,12 @@
        (set! (locsig-type) *clm-locsig-type*)
        (set! *mus-array-print-length* *clm-array-print-length*)
        (set! *auto-update-interval* 0.0) 
-       (if (eq? clipped 'unset)
-	   (if (and (or scaled-by scaled-to)
-		    (member sample-type (list mus-bfloat mus-lfloat mus-bdouble mus-ldouble)))
-	       (set! (mus-clipping) #f)
-	       (set! (mus-clipping) *clm-clipped*))
-	   (set! (mus-clipping) clipped))
+       (set! (mus-clipping) 
+	     (if (not (eq? clipped 'unset))
+		 clipped
+		 (and (not (and (or scaled-by scaled-to)
+				(member sample-type (list mus-bfloat mus-lfloat mus-bdouble mus-ldouble))))
+		      *clm-clipped*)))
        (set! *clm-srate* srate))
 
      (lambda ()
@@ -169,51 +167,51 @@
 		 (set! *reverb* reverb-1))))
 
        (let ((start (if statistics (get-internal-real-time)))
-	     (flush-reverb #f)
-	     (cycles 0)
 	     (revmax #f))
-	 (catch 'mus-error
-
-		(lambda ()
-		  (catch 'with-sound-interrupt
-			 thunk
-			 (lambda args 
-			   (snd-print (format #f "with-sound interrupted: ~{~A~^ ~}" (cdr args)))
-			   (set! flush-reverb #t)
-			   args)))
-
-		(lambda args
-		  ;; hit mus-error, for example:
-		  ;;   (with-sound () (fm-violin 0 1 440 .1 :amp-env '(0 0 1 1 1 2 3 0)))
-
-		  ;; user might have listener closed, or no listener so...
-		  (format () ";~%with-sound mus-error: ~{~A~^ ~}~%" (cdr args))
-
-		  ;; now try to get something to listener, since there may be no stdout
-		  (snd-print (format #f ";~%with-sound mus-error: ~{~A~^ ~}~%" (cdr args)))
-		  (set! flush-reverb #t)))
-		  
-	 (if (and reverb 
-		  (not flush-reverb)) ; i.e. not interrupted by error and trying to jump out
-	     (begin
-	       (if reverb-to-file
-		   (mus-close *reverb*))
-	       (if (and statistics 
-			(or reverb-to-file
-			    (vector? reverb-1)))
-		   (set! revmax (maxamp reverb-1)))
-	       (if reverb-to-file
-		   (set! *reverb* (make-file->sample reverb-1)))
-	       (apply reverb reverb-data)                                   ; here is the reverb call(!)
-	       (if reverb-to-file
-		   (mus-close *reverb*))
-	       (if (and reverb-to-file *clm-delete-reverb*)
-		   (delete-file reverb-1))))
+	 (let ((flush-reverb #f))
+	   (catch 'mus-error
+	     
+	     (lambda ()
+	       (catch 'with-sound-interrupt
+		 thunk
+		 (lambda args 
+		   (snd-print (format #f "with-sound interrupted: ~{~A~^ ~}" (cdr args)))
+		   (set! flush-reverb #t)
+		   args)))
+	     
+	     (lambda args
+	       ;; hit mus-error, for example:
+	       ;;   (with-sound () (fm-violin 0 1 440 .1 :amp-env '(0 0 1 1 1 2 3 0)))
+	       
+	       ;; user might have listener closed, or no listener so...
+	       (format () ";~%with-sound mus-error: ~{~A~^ ~}~%" (cdr args))
+	       
+	       ;; now try to get something to listener, since there may be no stdout
+	       (snd-print (format #f ";~%with-sound mus-error: ~{~A~^ ~}~%" (cdr args)))
+	       (set! flush-reverb #t)))
+	   
+	   (if (and reverb 
+		    (not flush-reverb)) ; i.e. not interrupted by error and trying to jump out
+	       (begin
+		 (if reverb-to-file
+		     (mus-close *reverb*))
+		 (if (and statistics 
+			  (or reverb-to-file
+			      (vector? reverb-1)))
+		     (set! revmax (maxamp reverb-1)))
+		 (if reverb-to-file
+		     (set! *reverb* (make-file->sample reverb-1)))
+		 (apply reverb reverb-data)                                   ; here is the reverb call(!)
+		 (when reverb-to-file
+		   (mus-close *reverb*)
+		   (if *clm-delete-reverb*
+		       (delete-file reverb-1))))))
 
 	 (if output-to-file
 	     (mus-close *output*))
 
 	 (let ((snd-output #f)
+	       (cycles 0)
 	       (cur-sync #f))
 	   (if statistics
 	       (set! cycles (* 1.0 (- (get-internal-real-time) start))))
@@ -221,12 +219,12 @@
 	   (if (and to-snd output-to-file)
 	       (let ((cur (find-sound output-1)))
 		 (set! cur-sync (and cur (sync cur)))
-		 (if cur 
-		     (set! snd-output (update-sound cur))
-		     (if (= header-type mus-raw)
-			 (set! snd-output (open-raw-sound output-1 channels (floor srate) sample-type))
-			 ;; open-sound here would either ask for raw settings or use possibly irrelevant defaults
-			 (set! snd-output (open-sound output-1))))
+		 (set! snd-output (if cur 
+				      (update-sound cur)
+				      (if (= header-type mus-raw)
+					  (open-raw-sound output-1 channels (floor srate) sample-type)
+					  ;; open-sound here would either ask for raw settings or use possibly irrelevant defaults
+					  (open-sound output-1))))
 		 (set! (sync snd-output) #t)))
 
 	   (if statistics
@@ -247,59 +245,63 @@
 			(if output-to-file
 			    (if to-snd
 				(maxamp snd-output #t) ; this is a list of chan maxs '(.1 .2)
-				(let ((lst (mus-sound-maxamp output-1)))
-				  (do ((i 0 (+ i 2)))
-				      ((>= i (length lst)))
-				    (set! (lst i) (/ (lst i) *clm-srate*)))
-				  lst))
+				(do ((lst (mus-sound-maxamp output-1))
+				     (i 0 (+ i 2)))
+				      ((>= i (length lst))
+				       lst)
+				  (set! (lst i) (/ (lst i) *clm-srate*))))
 			    (if (vector? output-1)
 				(list (maxamp output-1))
 				'(0.0)))
 			(if revmax (format #f "  rev max: ~,4F~%" revmax) "")
 			cycles)))
 
-	   (if (or scaled-to scaled-by)
-	       (cond (output-to-file
-		      (let* ((scale-output (or snd-output (open-sound output-1)))
-			     (old-sync (sync scale-output)))
-			(set! (sync scale-output) (+ (sync-max) 1))          ; make sure scaling doesn't follow sync
-			(if scaled-to
-			    (scale-to scaled-to scale-output)
-			    (scale-by scaled-by scale-output))
-			(set! (sync scale-output) old-sync)
-			(save-sound scale-output)
-			(if (not to-snd) 
-			    (close-sound scale-output))))
-		     ((float-vector? output-1)
-		      (if scaled-to
-			  (let ((pk (float-vector-peak output-1)))
-			    (if (> pk 0.0)
-				(float-vector-scale! output-1 (/ scaled-to pk))))
-			  (float-vector-scale! output-1 scaled-by)))
-		     ((vector? output-1)
-		      (if scaled-to
-			  (let ((pk (maxamp output-1)))
-			    (if (> pk 0.0)
-				(let ((scl (/ scaled-to pk)))
-				  (do ((i 0 (+ i 1)))
-				      ((= i (length output-1)))
-				    (set! (output-1 i) (* scl (output-1 i)))))))
-			  (do ((i 0 (+ i 1)))
-			      ((= i (length output-1)))
-			    (set! (output-1 i) (* scaled-by (output-1 i))))))))
+	   (cond ((not (or scaled-to scaled-by)))
+
+		 (output-to-file
+		  (let* ((scale-output (or snd-output (open-sound output-1)))
+			 (old-sync (sync scale-output)))
+		    (set! (sync scale-output) (+ (sync-max) 1))          ; make sure scaling doesn't follow sync
+		    (if scaled-to
+			(scale-to scaled-to scale-output)
+			(scale-by scaled-by scale-output))
+		    (set! (sync scale-output) old-sync)
+		    (save-sound scale-output)
+		    (if (not to-snd) 
+			(close-sound scale-output))))
+		 
+		 ((float-vector? output-1)
+		  (if scaled-to
+		      (let ((pk (float-vector-peak output-1)))
+			(if (> pk 0.0)
+			    (float-vector-scale! output-1 (/ scaled-to pk))))
+		      (float-vector-scale! output-1 scaled-by)))
+		 
+		 ((not (vector? output-1)))
+		 
+		 (scaled-to
+		  (let ((pk (maxamp output-1)))
+		    (if (> pk 0.0)
+			(do ((scl (/ scaled-to pk))
+			     (i 0 (+ i 1)))
+			    ((= i (length output-1)))
+			  (set! (output-1 i) (* scl (output-1 i)))))))
+		 (else
+		  (do ((i 0 (+ i 1)))
+		      ((= i (length output-1)))
+		    (set! (output-1 i) (* scaled-by (output-1 i))))))
 	   
-	   (if (and play output-to-file)
+	   (when output-to-file
+	     (when play
 	       (if to-snd
-		   (if *clm-player*
-		       (*clm-player* snd-output)
-		       (*default-player* snd-output))
+		   ((or *clm-player* *default-player*) snd-output)
 		   (*default-player* output-1)))
 
-	   (if (and to-snd output-to-file)
-	       (begin
-		 (update-time-graph snd-output)
-		 (goto-listener-end)
-		 (if (number? cur-sync) (set! (sync snd-output) cur-sync)))))
+	     (when to-snd
+	       (update-time-graph snd-output)
+	       (goto-listener-end)
+	       (if (number? cur-sync) (set! (sync snd-output) cur-sync)))))
+
 	 output-1))
 
      (lambda () 
@@ -375,10 +377,10 @@
 
 (define (with-mixed-sound-mix-info id snd)
 
-  (define (find-if pred l)
-    (cond ((null? l) #f)
-	  ((pred (car l)) (car l))
-	  (else (find-if pred (cdr l)))))
+  (define (find-if pred lst)
+    (cond ((null? lst) #f)
+	  ((pred (car lst)) (car lst))
+	  (else (find-if pred (cdr lst)))))
 
   (let ((all-info (sound-property 'with-mixed-sound-info snd)))
     ;; each entry is '(mx-id beg chans note)
@@ -482,12 +484,12 @@
 	     (let ((info (with-mixed-sound-mix-info id snd)))
 	       (if info
 		   (let ((call (cadddr info)))
-		     (if (not (= (cadr info) (mix-position id)))
-			 (format oput "  (~A ~,3F~{ ~A~})~%"
-				 (car call) 
-				 (/ (mix-position id) (* 1.0 (srate snd)))
-				 (cddr call))
-			 (format oput "  ~A~%" call)))
+		     (format oput (if (= (cadr info) (mix-position id))
+				      (values "  ~A~%" call)
+				      (values "  (~A ~,3F~{ ~A~})~%"
+					      (car call) 
+					      (/ (mix-position id) (* 1.0 (srate snd)))
+					      (cddr call)))))
 		   (status-report "can't find note associated with mix ~A" id))))
 	   cur-mixes)
 	  (format oput ")~%")
@@ -630,55 +632,50 @@
 	  start)))
 
 (define (finish-with-sound wsd)
-  (if (eq? (car wsd) 'with-sound-data)
-      (let ((cycles 0)
-	    (output (wsd 1))
-	    (reverb (wsd 2))
-	    (revfile (wsd 3))
+  (if (not (eq? (car wsd) 'with-sound-data))
+      (error 'wrong-type-arg (list "finish-with-sound" wsd))
+      (let ((output (wsd 1))
 	    (old-srate (wsd 4))
 	    (statistics (wsd 5))
 	    (to-snd (wsd 6))
 	    (scaled-to (wsd 7))
 	    (scaled-by (wsd 8))
 	    (play (wsd 9))
-	    (reverb-data (wsd 10))
 	    (start (wsd 11)))
 
-	(if reverb
-	    (begin
-	      (mus-close *reverb*)
-	      (if (string? revfile)
-		  (set! *reverb* (make-file->sample revfile))
-		  (set! *reverb* revfile))
-	      (apply reverb reverb-data)
-	      (mus-close *reverb*)))
+	(let ((reverb (wsd 2))
+	      (revfile (wsd 3))
+	      (reverb-data (wsd 10)))
+	  (if reverb
+	      (begin
+		(mus-close *reverb*)
+		(set! *reverb* (if (string? revfile)
+				   (make-file->sample revfile)
+				   revfile))
+		(apply reverb reverb-data)
+		(mus-close *reverb*))))
+
 	(if (mus-output? *output*)
 	    (mus-close *output*))
-
-	(if statistics
-	    (set! cycles (/ (- (get-internal-real-time) start) 100)))
 	(if (and to-snd (string? output))
 	    (let ((snd-output (open-sound output)))
 	      (set! (sync snd-output) #t)
 	      (if statistics
-		  (snd-print 
-		   (format #f "~A:~%  maxamp: ~A,~%  compute time: ~A~%"
-			   output
-			   (maxamp snd-output #t)
-			   cycles)))
-		 (if (or scaled-to scaled-by)
-		     (begin
-		       (if scaled-to
-			   (scale-to scaled-to snd-output)
-			   (if scaled-by
-			       (scale-by scaled-by snd-output)))
-		       (save-sound snd-output)))
+		  (snd-print (format #f "~A:~%  maxamp: ~A,~%  compute time: ~A~%"
+				     output
+				     (maxamp snd-output #t)
+				     (/ (- (get-internal-real-time) start) 100))))
+	      (if (or scaled-to scaled-by)
+		  (begin
+		    (if scaled-to
+			(scale-to scaled-to snd-output)
+			(if scaled-by
+			    (scale-by scaled-by snd-output)))
+		    (save-sound snd-output)))
 	      (if play (*default-player* snd-output))
 	      (update-time-graph snd-output)))
 	(set! *clm-srate* old-srate)
-	output)
-      (error 'wrong-type-arg
-	     (list "finish-with-sound" wsd))))
+	output)))
 
 
 (define wsdat-play ; for cm
@@ -695,62 +692,53 @@
 (define ws-save-state 
   (let ((documentation "(ws-save-state filename) is an after-save-state-hook function that saves the current with-sound global settings"))
     (lambda (hook)
-      (let ((filename (hook 'name)))
+      (let ((fd (open-output-file (hook 'name) "a")))
+	;; should the save-state file load this file if it hasn't been loaded? (what path?)
+	(format fd "~%~%;;; from ws.scm~%")
+	(format fd "(if (defined? '*clm-srate*)~%")
+	(format fd "    (begin~%")
+	(format fd "      (set! *clm-srate* ~A)~%" *clm-srate*)
+	(format fd "      (set! *clm-file-name* ~S)~%" *clm-file-name*)
+	(format fd "      (set! *clm-channels* ~A)~%" *clm-channels*)
+	(format fd "      (set! *clm-sample-type* ~A)~%" (mus-sample-type->string *clm-sample-type*))
+	(format fd "      (set! *clm-header-type* ~A)))~%" (mus-header-type->string *clm-header-type*))
+	(close-output-port fd)))))
 
-	(define (open-appending filename)
-	  (open-output-file filename "a"))
-    
-	(define close-appending close-output-port)
-    
-	(let ((fd (open-appending filename)))
-	  ;; fd is a Scheme port at this point (not an integer), so we can use format etc
-	  ;; should the save-state file load this file if it hasn't been loaded? (what path?)
-	  (format fd "~%~%;;; from ws.scm~%")
-	  (format fd "(if (defined? '*clm-srate*)~%")
-	  (format fd "    (begin~%")
-	  (format fd "      (set! *clm-srate* ~A)~%" *clm-srate*)
-	  (format fd "      (set! *clm-file-name* ~S)~%" *clm-file-name*)
-	  (format fd "      (set! *clm-channels* ~A)~%" *clm-channels*)
-	  (format fd "      (set! *clm-sample-type* ~A)~%" (mus-sample-type->string *clm-sample-type*))
-	  (format fd "      (set! *clm-header-type* ~A)))~%" (mus-header-type->string *clm-header-type*))
-	  (close-appending fd))))))
-
-(if (not (member ws-save-state (hook-functions after-save-state-hook) eq?))
+(if (not (memq ws-save-state (hook-functions after-save-state-hook)))
     (set! (hook-functions after-save-state-hook) (list ws-save-state)))
 
 
 ;;; -------- ->frequency --------
 
 (define ->frequency
-  (let ((main-pitch (/ 440.0 (expt 2.0 (/ 57 12)))) ; a4 = 440Hz is pitch 57 in our numbering
-	(last-octave 0)                             ; octave number can be omitted
+  (let ((main-pitch (/ 440.0 (expt 2.0 19/4))) ;(/ 57 12) ; a4 = 440Hz is pitch 57 in our numbering
+	(last-octave 0)                                   ; octave number can be omitted
 	(ratios (vector 1.0 256/243 9/8 32/27 81/64 4/3 1024/729 3/2 128/81 27/16 16/9 243/128 2.0))
 	(documentation "(->frequency pitch pythagorean) returns the frequency (Hz) of the 'pitch', a CLM/CM style note name as a \
 symbol: 'e4 for example.  If 'pythagorean', the frequency calculation uses small-integer ratios, rather than equal-tempered tuning."))
     (lambda* (pitch pythagorean)          ; pitch can be pitch name or actual frequency
-      (if (symbol? pitch)
+      (if (not (symbol? pitch))
+	  pitch
 	  (let* ((name (string-downcase (symbol->string pitch)))
-		 (base-char (name 0))
-		 (sign-char (and (> (length name) 1)
-				 (not (char-numeric? (name 1)))
-				 (not (char=? (name 1) #\n))
-				 (name 1)))
 		 (octave-char (if (and (> (length name) 1)
 				       (char-numeric? (name 1))) 
 				  (name 1)
 				  (and (> (length name) 2) 
 				       (char-numeric? (name 2))
 				       (name 2))))
-		 (base (modulo (+ 5 (- (char->integer base-char) (char->integer #\a))) 7)) ; c-based (diatonic) octaves
-		 (sign (if (not sign-char) 0 (if (char=? sign-char #\f) -1 1)))
+		 (sign-char (and (> (length name) 1)
+				 (not (char-numeric? (name 1)))
+				 (not (char=? (name 1) #\n))
+				 (name 1)))
 		 (octave (if octave-char (- (char->integer octave-char) (char->integer #\0)) last-octave))
-		 (base-pitch (+ sign (case base ((0) 0) ((1) 2) ((2) 4) ((3) 5) ((4) 7) ((5) 9) ((6) 11))))
+		 (base-pitch (let ((base (modulo (- (+ (char->integer (name 0)) 5) (char->integer #\a)) 7)) ; c-based (diatonic) octaves	   
+				   (sign (if (not sign-char) 0 (if (char=? sign-char #\f) -1 1))))
+			       (+ sign (case base ((0) 0) ((1) 2) ((2) 4) ((3) 5) ((4) 7) ((5) 9) ((6) 11)))))
 		 (et-pitch (+ base-pitch (* 12 octave))))
 	    (set! last-octave octave)
-	    (if pythagorean
-		(* main-pitch (expt 2 octave) (ratios base-pitch))
-		(* main-pitch (expt 2.0 (/ et-pitch 12)))))
-	  pitch))))
+	    (* main-pitch (if pythagorean
+			      (* (expt 2 octave) (ratios base-pitch))
+			      (expt 2.0 (/ et-pitch 12)))))))))
 
 
 ;;; -------- ->sample --------
@@ -770,33 +758,32 @@ symbol: 'e4 for example.  If 'pythagorean', the frequency calculation uses small
 (define-macro (defgenerator struct-name . fields)
 
   (define (list->bindings lst)
-    (let ((len (length lst)))
-      (let ((nlst (make-list (* len 2))))
-	(do ((old lst (cdr old))
-	     (nsym nlst (cddr nsym)))
-	    ((null? old) nlst)
-	  (if (pair? (car old))
-	      (begin
-		(list-set! nsym 1 (caar old))
-		(list-set! nsym 0 (list 'quote (caar old))))
-	      (begin
-		(list-set! nsym 1 (car old))
-		(list-set! nsym 0 (list 'quote (car old)))))))))
+    (let ((nlst (make-list (* (length lst) 2))))
+      (do ((old lst (cdr old))
+	   (nsym nlst (cddr nsym)))
+	  ((null? old) nlst)
+	(if (pair? (car old))
+	    (begin
+	      (list-set! nsym 1 (caar old))
+	      (list-set! nsym 0 (list 'quote (caar old))))
+	    (begin
+	      (list-set! nsym 1 (car old))
+	      (list-set! nsym 0 (list 'quote (car old))))))))
 
-  (let* ((name (if (pair? struct-name) 
-		   (car struct-name) 
-		   struct-name))
-	 (sname (if (string? name) 
-		    name 
-		    (symbol->string name)))
-	 (wrapper (or (and (pair? struct-name)
-			   (or (and (> (length struct-name) 2)
-				    (eq? (struct-name 1) :make-wrapper)
-				    (struct-name 2))
-			       (and (= (length struct-name) 5)
-				    (eq? (struct-name 3) :make-wrapper)
-				    (struct-name 4))))
-		      (lambda (gen) gen)))
+  (let* ((sname (let ((name (if (pair? struct-name) 
+				(car struct-name) 
+				struct-name)))
+		  (if (string? name) 
+		      name 
+		      (symbol->string name))))
+	 (wrapper (let ((wrap (and (pair? struct-name)
+				   (or (and (> (length struct-name) 2)
+					    (eq? (struct-name 1) :make-wrapper)
+					    (struct-name 2))
+				       (and (= (length struct-name) 5)
+					    (eq? (struct-name 3) :make-wrapper)
+					    (struct-name 4))))))
+		    (or wrap (lambda (gen) gen))))
 	 (methods (and (pair? struct-name)
 		       (or (and (> (length struct-name) 2)
 				(eq? (struct-name 1) :methods)
@@ -805,18 +792,18 @@ symbol: 'e4 for example.  If 'pythagorean', the frequency calculation uses small
 				(eq? (struct-name 3) :methods)
 				(struct-name 4))))))
     `(begin 
-       (define ,(string->symbol (string-append sname "?")) #f)
-       (define ,(string->symbol (string-append "make-" sname)) #f)
+       (define ,(symbol sname "?") #f)
+       (define ,(symbol "make-" sname) #f)
 
-       (let ((gen-type ',(string->symbol (string-append "+" sname "+")))
+       (let ((gen-type ',(symbol "+" sname "+"))
 	     (gen-methods (and ,methods (apply inlet ,methods))))
 	 
-	 (set! ,(string->symbol (string-append sname "?"))
+	 (set! ,(symbol sname "?")
 	       (lambda (obj)
 		 (and (let? obj)
 		      (eq? (obj 'mus-generator-type) gen-type))))
 
-	 (set! ,(string->symbol (string-append "make-" sname))
+	 (set! ,(symbol "make-" sname)
 	       (lambda* ,(map (lambda (n) 
 				(if (pair? n) n (list n 0.0)))
 			      fields)
@@ -884,7 +871,7 @@ symbol: 'e4 for example.  If 'pythagorean', the frequency calculation uses small
 
 
 (define (mix-notelists . notelists)
-  ;; assume the second parameter is the begin time in seconds (the 1st is the instrument name)
+  ;; assume the second parameter is the begin time in seconds (the first is the instrument name)
   (sort! 
    (apply append notelists)
    (lambda (note1 note2)
@@ -905,9 +892,9 @@ symbol: 'e4 for example.  If 'pythagorean', the frequency calculation uses small
   (let ((old-output *output*))
     (dynamic-wind
 	(lambda ()
-	  (if (string? output)
-	      (set! *output* (make-sample->file output channels sample-type header-type "with-simple-sound output"))
-	      (set! *output* output)))
+	  (set! *output* (if (string? output)
+			     (make-sample->file output channels sample-type header-type "with-simple-sound output")
+			     output)))
 	(lambda ()
 	  (thunk)
 	  output)
